# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ooi.tests import fakes
from ooi.tests.functional.middleware import test_middleware
from ooi import utils


def build_occi_volume(vol):
    name = vol["displayName"]
    vol_id = vol["id"]
    size = vol["size"]
    # TODO(enolfc): use proper status!
    status = "online"

    cats = []
    cats.append('storage; '
                'scheme="http://schemas.ogf.org/occi/infrastructure#"; '
                'class="kind"; '
                'title="storage resource"; '
                'rel="http://schemas.ogf.org/occi/core#resource"; '
                'location="%s/storage/"' % fakes.application_url)
    attrs = [
        'occi.core.title="%s"' % name,
        'occi.storage.size=%s' % size,
        'occi.storage.state="%s"' % status,
        'occi.core.id="%s"' % vol_id,
    ]
    links = []
    links.append('<%s/storage/%s?action=backup>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/storage/action#backup"' %
                 (fakes.application_url, vol_id))
    links.append('<%s/storage/%s?action=resize>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/storage/action#resize"' %
                 (fakes.application_url, vol_id))
    links.append('<%s/storage/%s?action=online>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/storage/action#online"' %
                 (fakes.application_url, vol_id))
    links.append('<%s/storage/%s?action=snapshot>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/storage/action#snapshot"' %
                 (fakes.application_url, vol_id))
    links.append('<%s/storage/%s?action=offline>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/storage/action#offline"' %
                 (fakes.application_url, vol_id))

    result = []
    for c in cats:
        result.append(("Category", c))
    for l in links:
        result.append(("Link", l))
    for a in attrs:
        result.append(("X-OCCI-Attribute", a))
    return result


class TestStorageController(test_middleware.TestMiddleware):
    """Test OCCI storage controller."""

    def test_list_vols_empty(self):
        tenant = fakes.tenants["bar"]
        app = self.get_app()

        for url in ("/storage/", "/storage"):
            req = self._build_req(url, tenant["id"], method="GET")

            req.environ["HTTP_X_PROJECT_ID"] = tenant["id"]

            resp = req.get_response(app)

            expected_result = ""
            self.assertContentType(resp)
            self.assertExpectedResult(expected_result, resp)
            self.assertEqual(204, resp.status_code)

    def test_list_vols(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for url in ("/storage/", "/storage"):
            req = self._build_req(url, tenant["id"], method="GET")

            resp = req.get_response(app)

            self.assertEqual(200, resp.status_code)
            expected = []
            for s in fakes.volumes[tenant["id"]]:
                expected.append(
                    ("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "storage/%s" % s["id"]))
                )
            self.assertExpectedResult(expected, resp)

    def test_show_vol(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for volume in fakes.volumes[tenant["id"]]:
            req = self._build_req("/storage/%s" % volume["id"],
                                  tenant["id"], method="GET")

            resp = req.get_response(app)
            expected = build_occi_volume(volume)
            self.assertContentType(resp)
            self.assertExpectedResult(expected, resp)
            self.assertEqual(200, resp.status_code)

    def test_vol_not_found(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        req = self._build_req("/storage/%s" % uuid.uuid4().hex,
                              tenant["id"], method="GET")
        resp = req.get_response(app)
        self.assertEqual(404, resp.status_code)

    def test_create_vol_no_size(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        headers = {
            'Category': (
                'storage;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind"')
        }
        req = self._build_req("/storage", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        self.assertEqual(400, resp.status_code)
        self.assertDefaults(resp)

    def test_create_vol(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        headers = {
            'Category': (
                'storage;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind"'),
            'X-OCCI-Attribute': (
                'occi.storage.size=1'
            )
        }
        req = self._build_req("/storage", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        expected = [("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "storage/%s" % "foo"))]
        self.assertEqual(200, resp.status_code)
        self.assertExpectedResult(expected, resp)
        self.assertDefaults(resp)

    def test_delete_vol(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for volume in fakes.volumes[tenant["id"]]:
            req = self._build_req("/storage/%s" % volume["id"],
                                  tenant["id"], method="DELETE")
            resp = req.get_response(app)
            self.assertContentType(resp)
            self.assertEqual(204, resp.status_code)

    # TODO(enolfc): find a way to be sure that all volumes
    #               are in fact deleted.
    def test_delete_all_vols(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        req = self._build_req("/storage/", tenant["id"], method="DELETE")
        resp = req.get_response(app)
        self.assertContentType(resp)
        self.assertEqual(204, resp.status_code)

    def test_action_vol(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for action in ("online", "offline", "backup", "snapshot", "resize"):
            headers = {
                'Category': (
                    '%s;'
                    'scheme="http://schemas.ogf.org/occi/infrastructure/'
                    'storage/action#";'
                    'class="action"' % action)
            }
            for vol in fakes.volumes[tenant["id"]]:
                req = self._build_req("/storage/%s?action=%s" % (vol["id"],
                                                                 action),
                                      tenant["id"], method="POST",
                                      headers=headers)
                resp = req.get_response(app)
                self.assertDefaults(resp)
                self.assertEqual(501, resp.status_code)


class StorageControllerTextPlain(test_middleware.TestMiddlewareTextPlain,
                                 TestStorageController):
    """Test OCCI compute controller with Accept: text/plain."""


class StorageControllerTextOcci(test_middleware.TestMiddlewareTextOcci,
                                TestStorageController):
    """Test OCCI compute controller with Accept: text/occi."""



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
# Copyright 2015 LIP - INDIGO-DataCloud
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.api import base
from ooi.api import helpers
from ooi import exception
from ooi.occi.core import collection
from ooi.occi.infrastructure import compute
from ooi.occi.infrastructure import network
from ooi.occi.infrastructure import network_link
from ooi.occi import validator as occi_validator
from ooi.openstack import helpers as os_helpers
from ooi.openstack import network as os_network


def _get_network_link_resources(link_list):
    """Create OCCI networkLink instances from json format

    :param link_list: provides by the cloud infrastructure
    """
    occi_network_resources = []
    if link_list:
        for l in link_list:
            compute_id = l['compute_id']
            mac = l.get('mac', None)
            net_pool = l.get('pool', None)
            ip = l.get('ip', None)
            state = l.get('state', None)
            ip_id = l.get('ip_id', None)
            net_id = l['network_id']
            n = network.NetworkResource(title="network",
                                        id=net_id)
            c = compute.ComputeResource(title="Compute",
                                        id=compute_id
                                        )
            iface = os_network.OSNetworkInterface(c, n, mac, ip,
                                                  pool=net_pool,
                                                  ip_id=ip_id,
                                                  state=state)
            occi_network_resources.append(iface)
    return occi_network_resources


class Controller(base.Controller):
    def __init__(self, *args, **kwargs):
        super(Controller, self).__init__(*args, **kwargs)
        self.os_helper = helpers.OpenStackHelper(
            self.app,
            self.openstack_version
        )

    def index(self, req):
        """List NetworkLinks

        :param req: request object
        """
        link_list = self.os_helper.list_compute_net_links(req)
        occi_link_resources = _get_network_link_resources(link_list)
        return collection.Collection(resources=occi_link_resources)

    def _get_interface_from_id(self, req, id):
        """Get interface from id

        :param req: request object
        :param id: network link identification
        """
        try:
            server_id, network_id, server_addr = id.split('_', 2)
        except ValueError:
            raise exception.LinkNotFound(link_id=id)
        try:
            link = self.os_helper.get_compute_net_link(
                req,
                server_id,
                network_id,
                server_addr)
            occi_instance = _get_network_link_resources([link])[0]
        except Exception:
            raise exception.LinkNotFound(link_id=id)
        return occi_instance

    def show(self, req, id):
        """Get networkLink details

        :param req: request object
        :param id: networkLink identification
        """
        occi_instance = self._get_interface_from_id(req, id)

        return occi_instance

    def create(self, req, body=None):
        """Create a networkLink

        Creates a link between a server and a network.
        It could be fixed or floating IP.

        :param req: request object
        :param body: body request (not used)
        """
        parser = req.get_parser()(req.headers, req.body)
        scheme = {
            "category": network_link.NetworkInterface.kind,
            "optional_mixins": [
                os_network.OSFloatingIPPool,
            ]
        }
        obj = parser.parse()
        validator = occi_validator.Validator(obj)
        validator.validate(scheme)

        attrs = obj.get("attributes", {})
        _, net_id = helpers.get_id_with_kind(
            req,
            attrs.get("occi.core.target"),
            network.NetworkResource.kind)
        _, server_id = helpers.get_id_with_kind(
            req,
            attrs.get("occi.core.source"),
            compute.ComputeResource.kind)
        pool = None
        if os_network.OSFloatingIPPool.scheme in obj["schemes"]:
                pool = (
                    obj["schemes"][os_network.OSFloatingIPPool.scheme][0]
                )
        # Allocate public IP and associate it ot the server
        if net_id == os_helpers.PUBLIC_NETWORK:
            os_link = self.os_helper.assign_floating_ip(
                req, net_id, server_id, pool
            )
        else:
            # Allocate private network
            os_link = self.os_helper.create_port(
                req, net_id, server_id)
        occi_link = _get_network_link_resources([os_link])
        return collection.Collection(resources=occi_link)

    def delete(self, req, id):
        """Delete networks link

        :param req: current request
        :param id: identification
        """
        iface = self._get_interface_from_id(req, id)
        server = iface.source.id
        if iface.target.id == os_helpers.PUBLIC_NETWORK:
            # remove floating IP
            self.os_helper.remove_floating_ip(req, server,
                                              iface.address)

            # release IP
            self.os_helper.release_floating_ip(req,
                                               iface.ip_id)
        else:
            self.os_helper.delete_port(
                req, server, iface.ip_id)
        return []


# -*- coding: utf-8 -*-

# Copyright 2015 LIP - INDIGO-DataCloud
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.api import helpers
from ooi import exception
from ooi.log import log as logging
from ooi.openstack import helpers as os_helpers

LOG = logging.getLogger(__name__)


class OpenStackNeutron(helpers.BaseHelper):
    """Class to interact with the neutron API."""

    def __init__(self, neutron_endpoint):
        super(OpenStackNeutron, self).__init__(None, None)
        self.neutron_endpoint = neutron_endpoint

    @staticmethod
    def _build_link(net_id, compute_id, ip, mac=None, pool=None,
                    state='ACTIVE'):
        link = {}
        link['mac'] = mac
        link['pool'] = pool
        link['network_id'] = net_id
        link['compute_id'] = compute_id
        link['ip'] = ip
        link['state'] = os_helpers.network_status(state)
        return link

    @staticmethod
    def _build_networks(networks):
        ooi_net_list = []
        for net in networks:
            ooi_net = {}
            status = net.get("status", None)
            ooi_net["state"] = os_helpers.network_status(status)
            public = net.get('router:external', None)
            if public:
                ooi_net["id"] = os_helpers.PUBLIC_NETWORK
                # TODO(jorgesece): include info about pools
            else:
                ooi_net["id"] = net["id"]
            ooi_net["name"] = net.get("name", None)
            if "subnet_info" in net:
                sub = net["subnet_info"]
                ooi_net["address"] = sub.get("cidr", None)
                ooi_net["ip_version"] = sub.get("ip_version", None)
                ooi_net["gateway"] = sub.get("gateway_ip", None)
            ooi_net_list.append(ooi_net)
        return ooi_net_list

    def _get_public_network(self, req):
        """Get public network

        This method get public network id

        :param req: the incoming request
        """
        att_public = {"router:external": True}
        net_public = self.list_resources(req,
                                         'networks',
                                         att_public)
        return net_public[0]["id"]

    def list_resources(self, req, resource, parameters=None):
        """List resources.

        It returns json code from the server

        :param req: the incoming request
        :param resource: network resource to manage
        :param parameters: query parameters
        :param tenant: include tenant in the query parameters
        """
        path = "/%s" % resource
        os_req = self._make_get_request(req, path, parameters)
        response = os_req.get_response()
        return self.get_from_response(response, resource, [])

    def get_resource(self, req, resource, id):
        """Get information from a resource.

        :param req: the incoming request
        :param resource: network resource to manage
        :param id: subnet identification
        """
        path = "/%s/%s" % (resource, id)
        req = self._make_get_request(req, path)
        response = req.get_response()
        single_resource = resource[:-1]
        return self.get_from_response(response, single_resource, {})

    def create_resource(self, req, resource, parameters):
        """Create a resource.

        :param req: the incoming request
        :param resource: network resource to manage
        :param parameters: parameters with values for the new network
        """
        single_resource = resource[:-1]
        req_subnet = self._make_create_request(req, resource, parameters)
        response_subnet = req_subnet.get_response()
        json_response = self.get_from_response(
            response_subnet, single_resource, {})
        return json_response

    def delete_resource(self, req, resource, id):
        """Delete resource. It returns empty array

        :param req: the incoming request
        :param parameters: conain id
        """
        path = "/%s" % resource
        req = self._make_delete_request(req, path, id)
        response = req.get_response()
        return self.get_from_response(response, None, [])

    def _add_router_interface(self, req, router_id, subnet_id):
        """Add interface.

        :param req: the incoming request
        :param router_id: router identification
        :param subnet_id: router identification
        """
        path = "/routers/%s/add_router_interface" % router_id
        parameters = {'subnet_id': subnet_id}
        os_req = self._make_put_request(req, path, parameters)
        response = os_req.get_response()
        json_response = self.get_from_response(
            response, None, {})
        return json_response

    def _remove_router_interface(self, req, router_id, port_id):
        """Remove interface.

        :param req: the incoming request
        :param router_id: router identification
        :param subnet_id: router identification
        """
        path = "/routers/%s/remove_router_interface" % router_id
        parameters = {'port_id': port_id}
        os_req = self._make_put_request(req, path, parameters)
        response = os_req.get_response()
        json_response = self.get_from_response(
            response, None, {})
        return json_response

    def create_port(self, req, network_id, device_id):
        """Add a port to the subnet

        Returns the port information

        :param req: the incoming network
        :param network_id: network id
        :param device_id: device id
        """
        attributes_port = {
            'device_owner': 'compute:nova',
            'network_id': network_id,
            'device_id': device_id
        }
        p = self.create_resource(req,
                                 'ports',
                                 attributes_port)
        link = self._build_link(
            p["network_id"],
            p['device_id'],
            p["fixed_ips"][0]["ip_address"],
            mac=p["mac_address"],
            state=p["status"])
        return link

    def delete_port(self, req, mac):
        """Delete a port to the subnet

        Returns the port information

        :param req: the incoming network
        :param mac: link mac
        """
        attributes_port = {
            "mac_address": mac
        }
        ports = self.list_resources(
            req,
            'ports', attributes_port
        )
        if ports.__len__() == 0:
            raise exception.LinkNotFound(
                "Interface %s not found" % mac
            )
        out = self.delete_resource(req,
                                   'ports',
                                   ports[0]['id'])
        return out

    def get_network_id(self, req, mac, server_id=None):
        """Get the Network ID from the mac port

        :param req: the incoming network
        :param mac: mac port
        :param server_id: id not use in neutron
        """
        try:
            attributes_port = {
                "mac_address": mac
            }
            ports = self.list_resources(
                req,
                'ports', attributes_port
            )
            id = ports[0]['network_id']
        except Exception:
            raise exception.NetworkNotFound
        return id

    def _add_floating_ip(self, req, public_net_id, port_id):
        """Add floating to the public network and a port

        Creates the floating IP and asign it to the port
        of the device.

        :param req: the incoming network
        :param public_net_id: public network id
        :param port_id: port id of the device
        """
        attributes_port = {
            "floating_network_id": public_net_id,
            "port_id": port_id
        }
        floating_ip = self.create_resource(req,
                                           'floatingips',
                                           attributes_port)
        return floating_ip

    def _remove_floating_ip(self, req, public_net_id, ip):
        """Delete floating to the public network and a port

        Delete the floating IP and asign it to the port
        of the device.

        :param req: the incoming network
        :param public_net_id: network id
        :param ip: floating ip to remove
        """
        attributes_port = {
            "floating_network_id": public_net_id,
            "floating_ip_address": ip
        }
        try:
            floating_ip = self.list_resources(req,
                                              'floatingips',
                                              attributes_port)
            response = self.delete_resource(req,
                                            'floatingips',
                                            floating_ip[0]['id'])
        except Exception:
            raise exception.NotFound
        return response

    def get_network_details(self, req, id):
        """Get info from a network.

        It returns json code from the server

        :param req: the incoming network
        :param id: net identification
        """
        if id == os_helpers.PUBLIC_NETWORK:
            id = self._get_public_network(req)
        path = "/networks/%s" % id
        req = self._make_get_request(req, path)
        response = req.get_response()
        net = self.get_from_response(response, "network", {})
        # subnet
        if "subnets" in net:
            path = "/subnets/%s" % net["subnets"][0]
            req_subnet = self._make_get_request(req, path)
            response_subnet = req_subnet.get_response()
            net["subnet_info"] = self.get_from_response(
                response_subnet, "subnet", {})

        ooi_networks = self._build_networks([net])

        return ooi_networks[0]

    def list_networks(self, req):
        """List networks.

        It returns json code from the server

        :param req: the incoming request
        :param parameters: query parameters
        """
        networks = self.list_resources(req,
                                       'networks'
                                       )
        ooi_networks = self._build_networks(networks)
        return ooi_networks

    def create_network(self, req, name, cidr,
                       gateway=None, ip_version=None):
        """Create a full neutron network.

        It creates a private network conected to the public one.
        It creates a full network objects stack:
        network, subnet, port, and router.
        In case of error, the objects already created are deleted.

        :param req: the incoming request
        :param name: network resource to manage
        :param cidr: parameters with values
        :param gateway: gateway ip
        :param ip_version: ip version
        """
        # NETWORK
        net_param = {'name': name}
        net = self.create_resource(req,
                                   'networks',
                                   net_param)
        # SUBNETWORK
        try:
            if not ip_version:
                ip_version = 4
            subnet_param = {'network_id': net["id"],
                            'cidr': cidr,
                            'ip_version': ip_version
                            }
            if gateway:
                subnet_param.update(
                    {'gateway_ip': gateway}
                )

            if "ip_version" not in subnet_param:
                subnet_param['ip_version'] = 4
            net["subnet_info"] = self.create_resource(
                req, 'subnets', subnet_param)

        # INTERFACE and ROUTER information is agnostic to the user
            net_public = self._get_public_network(req)
            attributes_router = {"external_gateway_info": {
                "network_id": net_public}
            }
            router = self.create_resource(req,
                                          'routers',
                                          attributes_router)
            try:
                # create interface to the network
                self._add_router_interface(req,
                                           router['id'],
                                           net['subnet_info']['id']
                                           )
            except Exception as ex:
                self.delete_resource(req,
                                     'routers',
                                     router['id']
                                     )
                raise ex
        except Exception as ex:
            self.delete_resource(req,
                                 'networks', net['id'])
            raise ex
        ooi_net = self._build_networks([net])
        return ooi_net[0]

    def delete_network(self, req, id):
        """Delete a full network.

        :param req: the incoming request
        :param id: net identification
        """
        param = {"network_id": id}
        ports = self.list_resources(req, 'ports', param)
        for port in ports:
            if port['device_owner'] == "network:router_interface":
                self._remove_router_interface(req,
                                              port['device_id'],
                                              port['id'],
                                              )
                self.delete_resource(req,
                                     'routers', port["device_id"])
            else:
                self.delete_resource(req,
                                     'ports', port["id"])
        response = self.delete_resource(req,
                                        'networks',
                                        id)
        return response

    def assign_floating_ip(self, req, device_id):
        """assign floating ip to a server

        :param req: the incoming request
        :param device_id: device id
        """
        # net_id it is not needed if
        # there is just one port of the VM
        attributes_port = {'device_id': device_id}
        try:
            net_public = self._get_public_network(req)
        except Exception:
            raise exception.NetworkNotFound()
        try:
            ports = self.list_resources(req, 'ports', attributes_port)
            port_id = ports[0]['id']
            # subnet_id = ports[0]['fixed_ips'][0]['subnet_id']
        except Exception:
            raise exception.NotFound()
        response = self._add_floating_ip(req, net_public, port_id)
        try:
            link_public = self._build_link(
                ports[0]['network_id'],
                attributes_port['device_id'],
                response['floating_ip_address'],
                pool=response['floating_network_id'])
        except Exception:
            raise exception.OCCIInvalidSchema()
        return link_public

    def release_floating_ip(self, req, iface):
        """release floating ip from a server

        :param req: the incoming request
        :param iface: link information
        """
        # net_id it is not needed if there is just one port of the VM
        try:
            net_public = self._get_public_network(req)
        except Exception:
            raise exception.NetworkNotFound()
        response = self._remove_floating_ip(req, net_public, iface['ip'])

        return response

    def list_compute_net_links(self, req, network_id,
                               device_id):
        """List the network and compute links

        It lists every private and public ip related to
        the servers of the tenant

        :param req: the incoming request
        :param network_id: id network
        :param device_id: id device
        """

        param_port = {'device_owner': 'compute:nova',
                      'device_id': device_id,
                      'network_id': network_id
                      }
        link_list = []
        try:
            ports = self.list_resources(req, 'ports', param_port)
            for port in ports:
                link_private = self._build_link(
                    port["network_id"],
                    port['device_id'],
                    port["fixed_ips"][0]["ip_address"],
                    mac=port["mac_address"],
                    state=os_helpers.network_status(port["status"]))
                link_list.append(link_private)
                # Query public links associated to the port
                floating_ips = self.list_resources(req,
                                                   'floatingips',
                                                   {"port_id": port['id']})
                for f_ip in floating_ips:
                    link_public = self._build_link(
                        port["network_id"],
                        port['device_id'],
                        f_ip['floating_ip_address'],
                        mac=port["mac_address"],
                        pool=f_ip['floating_network_id'])

                    link_list.append(link_public)
        except Exception:
            raise exception.NotFound()
        return link_list

    def get_compute_net_link(self, req, compute_id, network_id,
                             ip):
        """Get a specific network/server link

        It shows a specific link (either private or public ip)

        :param req: the incoming request
        :param compute_id: server id
        :param network_id: network id
        :param ip: ip connected
        """
        try:
            if network_id == os_helpers.PUBLIC_NETWORK:
                param = {'floating_ip_address': ip}
                flo_ips = self.list_resources(req,
                                              'floatingips',
                                              param)
                for f_ip in flo_ips:
                    link_public = self._build_link(
                        network_id,
                        compute_id,
                        f_ip['floating_ip_address'],
                        pool=f_ip['floating_network_id'])
                    return link_public
            else:
                param_ports = {'device_id': compute_id,
                               'network_id': network_id}
                ports = self.list_resources(req, 'ports', param_ports)
                for p in ports:
                    if ip == p["fixed_ips"][0]["ip_address"]:
                        link_private = self._build_link(
                            p["network_id"],
                            p['device_id'],
                            p["fixed_ips"][0]["ip_address"],
                            mac=p["mac_address"],
                            state=p["status"])
                        return link_private
            raise exception.NotFound()
        except Exception:
            raise exception.NotFound()

    def run_action(self, req, action, net_id):
        """Run an action on a network.

        :param req: the incoming request
        :param action: the action to run
        :param net_id: server id to delete
        """
        os_req = self._make_action_reques(req, action, id)
        response = os_req.get_response()
        if response.status_int != 202:
            raise helpers.exception_from_response(response)



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from ooi.tests import fakes
from ooi.tests.functional.middleware import test_middleware


class TestQueryController(test_middleware.TestMiddleware):
    """Test OCCI query controller."""

    def test_query(self):
        tenant_id = fakes.tenants["bar"]["id"]
        result = self._build_req("/-/", tenant_id).get_response(self.get_app())
        self.assertDefaults(result)
        self.assertExpectedResult(fakes.fake_query_results(), result)
        self.assertEqual(200, result.status_code)

    def test_query_rfc(self):
        tenant_id = fakes.tenants["bar"]["id"]
        result = self._build_req("/.well-known/org/ogf/occi/-/",
                                 tenant_id).get_response(self.get_app())
        self.assertDefaults(result)
        self.assertExpectedResult(fakes.fake_query_results(), result)
        self.assertEqual(200, result.status_code)

    def test_query_rfc_equal(self):
        tenant_id = fakes.tenants["bar"]["id"]
        result_rfc = self._build_req("/.well-known/org/ogf/occi/-/",
                                     tenant_id).get_response(self.get_app())
        result_query = self._build_req("/-/",
                                       tenant_id).get_response(self.get_app())
        self.assertDefaults(result_rfc)
        self.assertDefaults(result_query)
        self.assertEqual(result_rfc.text, result_query.text)


class QueryControllerTextPlain(test_middleware.TestMiddlewareTextPlain,
                               TestQueryController):
    """Test OCCI query controller with Accept: text/plain."""


class QueryControllerTextOcci(test_middleware.TestMiddlewareTextOcci,
                              TestQueryController):
    """Test OCCI query controller with Accept: text/cci."""



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ooi.tests import fakes
from ooi.tests.functional.middleware import test_middleware
from ooi import utils


def build_occi_server(server):
    name = server["name"]
    server_id = server["id"]
    flavor_id = fakes.flavors[server["flavor"]["id"]]["id"]
    flavor_name = fakes.flavors[server["flavor"]["id"]]["name"]
    ram = fakes.flavors[server["flavor"]["id"]]["ram"]
    cores = fakes.flavors[server["flavor"]["id"]]["vcpus"]
    image_id = server["image"]["id"]

    status = server["status"].upper()
    if status in ("ACTIVE",):
        status = "active"
    elif status in ("PAUSED", "SUSPENDED", "STOPPED"):
        status = "suspended"
    else:
        status = "inactive"

    app_url = fakes.application_url
    cats = []
    cats.append('compute; '
                'scheme="http://schemas.ogf.org/occi/infrastructure#"; '
                'class="kind"; title="compute resource"; '
                'rel="http://schemas.ogf.org/occi/core#resource"; '
                'location="%s/compute/"' % app_url)
    cats.append('%s; '
                'scheme="http://schemas.openstack.org/template/os#"; '
                'class="mixin"; title="%s"; '
                'rel="http://schemas.ogf.org/occi/infrastructure#os_tpl"; '
                'location="%s/os_tpl/%s"'
                % (image_id, image_id, app_url, image_id)),
    cats.append('%s; '
                'scheme="http://schemas.openstack.org/template/resource#"; '
                'class="mixin"; title="Flavor: %s"; '
                'rel="http://schemas.ogf.org/occi/infrastructure#resource_tpl"'
                '; '
                'location="%s/resource_tpl/%s"'
                % (flavor_id, flavor_name, app_url, flavor_id)),

    attrs = [
        'occi.core.title="%s"' % name,
        'occi.compute.state="%s"' % status,
        'occi.compute.memory=%s' % ram,
        'occi.compute.cores=%s' % cores,
        'occi.compute.hostname="%s"' % name,
        'occi.core.id="%s"' % server_id,
    ]
    links = []
    links.append('<%s/compute/%s?action=restart>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/compute/action#restart"' %
                 (fakes.application_url, server_id))
    links.append('<%s/compute/%s?action=start>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/compute/action#start"' %
                 (fakes.application_url, server_id))
    links.append('<%s/compute/%s?action=stop>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/compute/action#stop"' %
                 (fakes.application_url, server_id))
    links.append('<%s/compute/%s?action=suspend>; '
                 'rel="http://schemas.ogf.org/occi/'
                 'infrastructure/compute/action#suspend"' %
                 (fakes.application_url, server_id))

    result = []
    for c in cats:
        result.append(("Category", c))
    for l in links:
        result.append(("Link", l))
    for a in attrs:
        result.append(("X-OCCI-Attribute", a))
    return result


class TestComputeController(test_middleware.TestMiddleware):
    """Test OCCI compute controller."""

    def test_list_vms_empty(self):
        tenant = fakes.tenants["bar"]
        app = self.get_app()

        for url in ("/compute/", "/compute"):
            req = self._build_req(url, tenant["id"], method="GET")

            req.environ["HTTP_X_PROJECT_ID"] = tenant["id"]

            resp = req.get_response(app)

            expected_result = ""
            self.assertDefaults(resp)
            self.assertExpectedResult(expected_result, resp)
            self.assertEqual(204, resp.status_code)

    def test_list_vms(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for url in ("/compute/", "/compute"):
            req = self._build_req(url, tenant["id"], method="GET")
            resp = req.get_response(app)

            self.assertEqual(200, resp.status_code)
            expected = []
            for s in fakes.servers[tenant["id"]]:
                expected.append(
                    ("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "compute/%s" % s["id"]))
                )
            self.assertDefaults(resp)
            self.assertExpectedResult(expected, resp)

    def test_show_vm(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for server in fakes.servers[tenant["id"]]:
            req = self._build_req("/compute/%s" % server["id"],
                                  tenant["id"], method="GET")

            resp = req.get_response(app)
            expected = build_occi_server(server)
            self.assertDefaults(resp)
            self.assertExpectedResult(expected, resp)
            self.assertEqual(200, resp.status_code)

    def test_vm_not_found(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        req = self._build_req("/compute/%s" % uuid.uuid4().hex,
                              tenant["id"], method="GET")
        resp = req.get_response(app)
        self.assertEqual(404, resp.status_code)

    def test_action_vm(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for action in ("stop", "start", "restart"):
            headers = {
                'Category': (
                    '%s;'
                    'scheme="http://schemas.ogf.org/occi/infrastructure/'
                    'compute/action#";'
                    'class="action"' % action)
            }
            for server in fakes.servers[tenant["id"]]:
                req = self._build_req("/compute/%s?action=%s" % (server["id"],
                                                                 action),
                                      tenant["id"], method="POST",
                                      headers=headers)
                resp = req.get_response(app)
                self.assertDefaults(resp)
                self.assertEqual(204, resp.status_code)

    def test_invalid_action(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        action = "foo"
        for server in fakes.servers[tenant["id"]]:
            req = self._build_req("/compute/%s?action=%s" % (server["id"],
                                                             action),
                                  tenant["id"], method="POST")
            resp = req.get_response(app)
            self.assertDefaults(resp)
            self.assertEqual(400, resp.status_code)

    def test_action_body_mismatch(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        action = "stop"
        headers = {
            'Category': (
                'start;'
                'scheme="http://schemas.ogf.org/occi/infrastructure/'
                'compute/action#";'
                'class="action"')
        }
        for server in fakes.servers[tenant["id"]]:
            req = self._build_req("/compute/%s?action=%s" % (server["id"],
                                                             action),
                                  tenant["id"], method="POST",
                                  headers=headers)
            resp = req.get_response(app)
            self.assertDefaults(resp)
            self.assertEqual(400, resp.status_code)

    def test_create_vm(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        headers = {
            'Category': (
                'compute;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind",'
                'foo;'
                'scheme="http://schemas.openstack.org/template/resource#";'
                'class="mixin",'
                'bar;'
                'scheme="http://schemas.openstack.org/template/os#";'
                'class="mixin"')
        }
        req = self._build_req("/compute", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        expected = [("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "compute/%s" % "foo"))]
        self.assertEqual(200, resp.status_code)
        self.assertExpectedResult(expected, resp)
        self.assertDefaults(resp)

    def test_create_vm_incomplete(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        headers = {
            'Category': (
                'compute;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind",'
                'bar;'
                'scheme="http://schemas.openstack.org/template/os#";'
                'class="mixin"')
        }

        req = self._build_req("/compute", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        self.assertEqual(400, resp.status_code)
        self.assertDefaults(resp)

    def test_create_with_context(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        headers = {
            'Category': (
                'compute;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind",'
                'foo;'
                'scheme="http://schemas.openstack.org/template/resource#";'
                'class="mixin",'
                'bar;'
                'scheme="http://schemas.openstack.org/template/os#";'
                'class="mixin",'
                'user_data;'
                'scheme="http://schemas.openstack.org/compute/instance#";'
                'class="mixin"'
            ),
            'X-OCCI-Attribute': (
                'org.openstack.compute.user_data="foo"'
            )
        }

        req = self._build_req("/compute", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        expected = [("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "compute/%s" % "foo"))]
        self.assertEqual(200, resp.status_code)
        self.assertExpectedResult(expected, resp)
        self.assertDefaults(resp)

    def test_create_vm_with_storage(self):
        tenant = fakes.tenants["foo"]
        target = utils.join_url(self.application_url + "/",
                                "storage/%s" % uuid.uuid4().hex)
        app = self.get_app()
        headers = {
            'Category': (
                'compute;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind",'
                'foo;'
                'scheme="http://schemas.openstack.org/template/resource#";'
                'class="mixin",'
                'bar;'
                'scheme="http://schemas.openstack.org/template/os#";'
                'class="mixin"'),
            'Link': (
                '</bar>;'
                'rel="http://schemas.ogf.org/occi/infrastructure#storage";'
                'occi.core.target="%s"') % target
        }
        req = self._build_req("/compute", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        expected = [("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "compute/%s" % "foo"))]
        self.assertEqual(200, resp.status_code)
        self.assertExpectedResult(expected, resp)
        self.assertDefaults(resp)

    def test_create_vm_with_network(self):
        tenant = fakes.tenants["foo"]
        target = utils.join_url(self.application_url + "/",
                                "network/%s" % uuid.uuid4().hex)
        app = self.get_app()
        headers = {
            'Category': (
                'compute;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind",'
                'foo;'
                'scheme="http://schemas.openstack.org/template/resource#";'
                'class="mixin",'
                'bar;'
                'scheme="http://schemas.openstack.org/template/os#";'
                'class="mixin"'),
            'Link': (
                '</bar>;'
                'rel="http://schemas.ogf.org/occi/infrastructure#network";'
                'occi.core.target="%s"') % target
        }
        req = self._build_req("/compute", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(app)

        expected = [("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "compute/%s" % "foo"))]
        self.assertEqual(200, resp.status_code)
        self.assertExpectedResult(expected, resp)
        self.assertDefaults(resp)

    def test_vm_links(self):
        tenant = fakes.tenants["baz"]

        app = self.get_app()

        for server in fakes.servers[tenant["id"]]:
            req = self._build_req("/compute/%s" % server["id"],
                                  tenant["id"], method="GET")

            resp = req.get_response(app)

            self.assertDefaults(resp)
            self.assertContentType(resp)
            self.assertEqual(200, resp.status_code)

            source = utils.join_url(self.application_url + "/",
                                    "compute/%s" % server["id"])
            # volumes
            vols = server.get("os-extended-volumes:volumes_attached", [])
            for v in vols:
                vol_id = v["id"]
                link_id = '_'.join([server["id"], vol_id])

                target = utils.join_url(self.application_url + "/",
                                        "storage/%s" % vol_id)
                self.assertResultIncludesLink(link_id, source, target, resp)

            # network
            addresses = server.get("addresses", {})
            for addr_set in addresses.values():
                for addr in addr_set:
                    ip = addr["addr"]
                    if addr["OS-EXT-IPS:type"] == "fixed":
                        net_id = fakes.ports[tenant["id"]][0]["net_id"]
                    else:
                        net_id = "PUBLIC"
                    link_id = '_'.join([server["id"], net_id, ip])
                    target = utils.join_url(self.application_url + "/",
                                            "network/%s" % net_id)
                    self.assertResultIncludesLink(link_id, source, target,
                                                  resp)

    def test_delete_vm(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        for s in fakes.servers[tenant["id"]]:
            req = self._build_req("/compute/%s" % s["id"],
                                  tenant["id"], method="DELETE")
            resp = req.get_response(app)
            self.assertContentType(resp)
            self.assertEqual(204, resp.status_code)

    def test_delete_vm_with_links(self):
        tenant = fakes.tenants["baz"]
        app = self.get_app()

        for s in fakes.servers[tenant["id"]]:
            req = self._build_req("/compute/%s" % s["id"],
                                  tenant["id"], method="DELETE")
            resp = req.get_response(app)
            self.assertContentType(resp)
            self.assertEqual(204, resp.status_code)

    # TODO(enolfc): find a way to be sure that all servers
    #               are in fact deleted.
    def test_delete_all_vms(self):
        tenant = fakes.tenants["foo"]
        app = self.get_app()

        req = self._build_req("/compute/", tenant["id"], method="DELETE")
        resp = req.get_response(app)
        self.assertContentType(resp)
        self.assertEqual(204, resp.status_code)


class ComputeControllerTextPlain(test_middleware.TestMiddlewareTextPlain,
                                 TestComputeController):
    """Test OCCI compute controller with Accept: text/plain."""


class ComputeControllerTextOcci(test_middleware.TestMiddlewareTextOcci,
                                TestComputeController):
    """Test OCCI compute controller with Accept: text/occi."""



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
# Copyright 2015 LIP - INDIGO-DataCloud
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ooi.tests import fakes
from ooi.tests.functional.middleware import test_middleware
from ooi import utils


class TestNetInterfaceController(test_middleware.TestMiddleware):
    """Test OCCI network interface controller."""

    def setUp(self):
        super(TestNetInterfaceController, self).setUp()
        self.application_url = fakes.application_url
        self.app = self.get_app()

    def test_list_ifaces_empty(self):
        tenant = fakes.tenants["bar"]

        for url in ("/networklink/", "/networklink"):
            req = self._build_req(url, tenant["id"], method="GET")

            req.environ["HTTP_X_PROJECT_ID"] = tenant["id"]

            resp = req.get_response(self.app)

            expected_result = ""
            self.assertContentType(resp)
            self.assertExpectedResult(expected_result, resp)
            self.assertEqual(204, resp.status_code)

    def test_list_ifaces(self):
        tenant = fakes.tenants["baz"]

        for url in ("/networklink/", "/networklink"):
            req = self._build_req(url, tenant["id"], method="GET")

            resp = req.get_response(self.app)

            self.assertEqual(200, resp.status_code)
            expected = []
            float_list = {}
            for floating_ip in fakes.floating_ips[tenant["id"]]:
                if floating_ip["instance_id"]:
                    float_list.update({floating_ip['fixed_ip']: floating_ip})
            instance_vm = fakes.linked_vm_id
            for p in fakes.ports[tenant["id"]]:
                for ip in p["fixed_ips"]:
                    link_id = '_'.join([instance_vm,
                                        p["net_id"],
                                        ip["ip_address"]])
                    expected.append(
                        ("X-OCCI-Location",
                         utils.join_url(self.application_url + "/",
                                        "networklink/%s" % link_id))
                    )
                    float_ip = float_list.get(ip['ip_address'], None)
                    if float_ip:
                        link_id = '_'.join([instance_vm,
                                            "PUBLIC",
                                            float_ip["ip"]])
                        expected.append(
                            ("X-OCCI-Location",
                             utils.join_url(self.application_url + "/",
                                            "networklink/%s" % link_id))
                        )

            self.assertExpectedResult(expected, resp)

    def test_show_iface(self):
        tenant = fakes.tenants["baz"]
        instance_vm = fakes.linked_vm_id
        for p in fakes.ports[tenant["id"]]:
            for ip in p["fixed_ips"]:
                link_id = '_'.join([instance_vm,
                                    p["net_id"],
                                    ip["ip_address"]]
                                   )
                req = self._build_req("/networklink/%s" % link_id,
                                      tenant["id"], method="GET")

                resp = req.get_response(self.app)
                self.assertContentType(resp)
                source = utils.join_url(self.application_url + "/",
                                        "compute/%s" % instance_vm)
                target = utils.join_url(self.application_url + "/",
                                        "network/%s" % p["net_id"])
                self.assertResultIncludesLinkAttr(link_id, source, target,
                                                  resp)
                self.assertEqual(200, resp.status_code)

    def test_show_invalid_id(self):
        tenant = fakes.tenants["foo"]
        link_id = uuid.uuid4().hex
        req = self._build_req("/networklink/%s" % link_id,
                              tenant["id"], method="GET")
        resp = req.get_response(self.app)
        self.assertEqual(404, resp.status_code)

    def test_create_link_invalid(self):
        tenant = fakes.tenants["foo"]
        net_id = fakes.ports[tenant['id']][0]['net_id']
        occi_net_id = utils.join_url(self.application_url + "/",
                                     "network/%s" % net_id)
        headers = {
            'Category': (
                'networkinterface;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind"'),
            'X-OCCI-Attribute': ('occi.core.source="foo", '
                                 'occi.core.target="%s"'
                                 ) % occi_net_id
        }
        req = self._build_req("/networklink", None, method="POST",
                              headers=headers)
        resp = req.get_response(self.app)
        self.assertEqual(400, resp.status_code)

    def test_create_link_no_pool(self):
        tenant = fakes.tenants["foo"]
        net_id = fakes.ports[tenant['id']][0]['net_id']
        occi_compute_id = utils.join_url(
            self.application_url + "/",
            "compute/%s" % fakes.linked_vm_id)
        occi_net_id = utils.join_url(self.application_url + "/",
                                     "network/%s" % net_id)
        headers = {
            'Category': (
                'networkinterface;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind"'),
            'X-OCCI-Attribute': ('occi.core.source="%s", '
                                 'occi.core.target="%s"'
                                 ) % (occi_compute_id, occi_net_id)
        }
        req = self._build_req("/networklink", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(self.app)
        self.assertEqual(200, resp.status_code)

    def test_create_link_with_pool(self):
        tenant = fakes.tenants["baz"]
        link_info = fakes.ports[tenant['id']][0]

        server_url = utils.join_url(self.application_url + "/",
                                    "compute/%s" % link_info['server_id'])
        net_url = utils.join_url(self.application_url + "/",
                                 "network/%s" % link_info['net_id'])
        pool_name = 'pool'
        headers = {
            'Category': ('networkinterface;'
                         'scheme="http://schemas.ogf.org/occi/'
                         'infrastructure#";'
                         'class="kind",'
                         '%s;'
                         'scheme="http://schemas.openstack.org/network/'
                         'floatingippool#"; class="mixin"') % pool_name,
            'X-OCCI-Attribute': ('occi.core.source="%s", '
                                 'occi.core.target="%s"'
                                 ) % (server_url, net_url)
        }
        req = self._build_req("/networklink", tenant["id"], method="POST",
                              headers=headers)
        resp = req.get_response(self.app)

        link_id = '_'.join([link_info['server_id'],
                            link_info['net_id'],
                            link_info['fixed_ips'][0]
                            ["ip_address"]])
        expected = [("X-OCCI-Location",
                     utils.join_url(self.application_url + "/",
                                    "networklink/%s" % link_id))]
        self.assertEqual(200, resp.status_code)
        self.assertExpectedResult(expected, resp)
        self.assertDefaults(resp)

    def test_delete_fixed(self):
        tenant = fakes.tenants["baz"]

        for n in fakes.ports[tenant["id"]]:
            if n["net_id"] != "PUBLIC":
                if n["server_id"]:
                    link_id = '_'.join([n["server_id"],
                                        n["net_id"],
                                        n["fixed_ips"]
                                        [0]["ip_address"]])
                    req = self._build_req(
                        "/networklink/%s" % link_id,
                        tenant["id"], method="DELETE")
                    resp = req.get_response(self.app)
                    self.assertContentType(resp)
                    self.assertEqual(204, resp.status_code)

    def test_delete_public(self):
        tenant = fakes.tenants["baz"]
        for n in fakes.floating_ips[tenant["id"]]:
            if n["instance_id"]:
                link_id = '_'.join([n["instance_id"],
                                    "PUBLIC",
                                    n["ip"]])
                req = self._build_req("/networklink/%s" % link_id,
                                      tenant["id"], method="DELETE")
                resp = req.get_response(self.app)
                self.assertContentType(resp)
                self.assertEqual(204, resp.status_code)

    def test_show_non_existant_compute(self):
        tenant = fakes.tenants["foo"]

        app = self.get_app()
        req = self._build_req("/networklink/%s_foo" % uuid.uuid4().hex,
                              tenant["id"], method="GET")
        resp = req.get_response(app)
        self.assertEqual(404, resp.status_code)

    def test_create_link_invalid_compute(self):
        app = self.get_app()
        net_id = utils.join_url(self.application_url + "/",
                                "network/floating")
        headers = {
            'Category': (
                'networkinterface;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind"'),
            'X-OCCI-Attribute': ('occi.core.source="foo", '
                                 'occi.core.target="%s"'
                                 ) % net_id
        }
        req = self._build_req("/networklink", None, method="POST",
                              headers=headers)
        resp = req.get_response(app)
        self.assertEqual(400, resp.status_code)

    def test_create_link_invalid_network(self):
        app = self.get_app()
        server_id = utils.join_url(self.application_url + "/",
                                   "compute/foo")
        headers = {
            'Category': (
                'networkinterface;'
                'scheme="http://schemas.ogf.org/occi/infrastructure#";'
                'class="kind"'),
            'X-OCCI-Attribute': ('occi.core.source="%s", '
                                 'occi.core.target="bar"'
                                 ) % server_id
        }
        req = self._build_req("/networklink", None, method="POST",
                              headers=headers)
        resp = req.get_response(app)
        self.assertEqual(400, resp.status_code)


class NetInterfaceControllerTextPlain(test_middleware.TestMiddlewareTextPlain,
                                      TestNetInterfaceController):
    """Test OCCI network link controller with Accept: text/plain."""


class NetInterfaceControllerTextOcci(test_middleware.TestMiddlewareTextOcci,
                                     TestNetInterfaceController):
    """Test OCCI network link controller with Accept: text/occi."""



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import webob.exc

import warnings

from ooi.log import log as logging

LOG = logging.getLogger(__name__)

warnings.simplefilter("default", DeprecationWarning)


def raise_deprecation_message(message):
    warnings.warn(message, DeprecationWarning, stacklevel=2)


class ConvertedException(webob.exc.WSGIHTTPException):
    def __init__(self, code=0, title="", explanation=""):
        self.code = code
        self.title = title
        self.explanation = explanation
        super(ConvertedException, self).__init__()


class OCCIException(Exception):
    """Base Nova Exception

    To correctly use this class, inherit from it and define
    a 'msg_fmt' property. That msg_fmt will get printf'd
    with the keyword arguments provided to the constructor.

    """
    msg_fmt = "An unknown exception occurred."
    code = 500
    headers = {}
    safe = False

    def __init__(self, message=None, **kwargs):
        self.kwargs = kwargs

        if 'code' not in self.kwargs:
            try:
                self.kwargs['code'] = self.code
            except AttributeError:
                pass

        if not message:
            try:
                message = self.msg_fmt % kwargs

            except Exception:
                # kwargs doesn't match a variable in the message
                # log the issue and the kwargs
                LOG.exception('Exception in string format operation')
                for name, value in kwargs.iteritems():
                    LOG.error("%s: %s" % (name, value))    # noqa

                message = self.msg_fmt

        super(OCCIException, self).__init__(message)

    def format_message(self):
        # NOTE(mrodden): use the first argument to the python Exception object
        # which should be our full NovaException message, (see __init__)
        return self.args[0]


class Invalid(OCCIException):
    msg_fmt = "Unacceptable parameters."
    code = 400


class InvalidAction(Invalid):
    msg_fmt = "Invalid action %(action)s provided."


class Forbidden(Invalid):
    msg_fmt = "Forbidden: %(reason)s"
    code = 403


class InvalidContentType(Invalid):
    msg_fmt = "Invalid Content-type %(content_type)s."
    code = 406


class NoContentType(InvalidContentType):
    msg_fmt = "No Content-type provided."


class InvalidAccept(InvalidContentType):
    msg_fmt = "Invalid Accept %(content_type)s."


class NotImplemented(OCCIException):
    msg_fmt = "Action not implemented."
    code = 501


class OCCIInvalidSchema(Invalid):
    msg_fmt = "Found invalid schema: '%(msg)s'."


class OCCIMissingType(Invalid):
    msg_fmt = "Missing OCCI types: '%(type_id)s'."


class OCCISchemaMismatch(Invalid):
    msg_fmt = ("Schema does not match. Expecting '%(expected)s', "
               "but found '%(found)s'.")


class NotFound(OCCIException):
    msg_fmt = "Not Found"
    code = 404


class LinkNotFound(NotFound):
    msg_fmt = "Link Not Found: '%(link_id)s"


class ResourceNotFound(NotFound):
    msg_fmt = "Resource Not Found: '%(resource_id)s'"


class NetworkNotFound(NotFound):
    msg_fmt = "Network Resource Not Found: '%(resource_id)s'"


class NetworkPoolFound(NotFound):
    msg_fmt = "Network Pool Not Found: '%(pool)s'"


class MissingKeypairName(Invalid):
    msg_fmt = "Missing Keypair Name"
    code = 400



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ooi.occi.infrastructure import compute
from ooi.occi.infrastructure import network
from ooi.occi.infrastructure import network_link
from ooi.occi.infrastructure import templates as occi_templates
from ooi.openstack import contextualization
from ooi.openstack import helpers
from ooi.openstack import network as os_network
from ooi.openstack import templates
from ooi.tests import base


class TestOpenStackOSTemplate(base.TestCase):
    def test_os_template(self):
        id = uuid.uuid4().hex
        title = "Frobble Image"
        location = "%s/%s" % (occi_templates.os_tpl._location, id)

        tpl = templates.OpenStackOSTemplate(id,
                                            title)
        self.assertEqual(id, tpl.term)
        self.assertEqual(title, tpl.title)
        self.assertTrue(tpl.scheme.startswith(helpers._PREFIX))
        self.assertIn(occi_templates.os_tpl, tpl.related)
        self.assertEqual(location, tpl.location)


class TestOpenStackResourceTemplate(base.TestCase):
    def test_resource_template(self):
        id = uuid.uuid4().hex
        name = "m1.humongous"
        cores = 10
        memory = 30
        disk = 40
        swap = 20
        ephemeral = 50
        location = "%s/%s" % (occi_templates.resource_tpl._location, id)

        tpl = templates.OpenStackResourceTemplate(id,
                                                  name,
                                                  cores,
                                                  memory,
                                                  disk,
                                                  swap=swap,
                                                  ephemeral=ephemeral)

        self.assertEqual(id, tpl.term)
        self.assertEqual("Flavor: %s" % name, tpl.title)
        self.assertTrue(tpl.scheme.startswith(helpers._PREFIX))
        self.assertIn(occi_templates.resource_tpl, tpl.related)
        self.assertEqual(cores, tpl.cores)
        self.assertEqual(memory, tpl.memory)
        self.assertEqual(disk, tpl.disk)
        self.assertEqual(swap, tpl.swap)
        self.assertEqual(ephemeral, tpl.ephemeral)
        self.assertEqual(name, tpl.name)
        self.assertEqual(location, tpl.location)


class TestHelpers(base.TestCase):
    def test_vm_state(self):
        self.assertEqual("active", helpers.vm_state("ACTIVE"))
        self.assertEqual("suspended", helpers.vm_state("SUSPENDED"))
        self.assertEqual("inactive", helpers.vm_state("PAUSED"))
        self.assertEqual("inactive", helpers.vm_state("STOPPED"))
        self.assertEqual("inactive", helpers.vm_state("BUILDING"))

    def test_vol_state(self):
        self.assertEqual("online", helpers.vol_state("in-use"))


class TestOpenStackUserData(base.TestCase):
    def test_os_userdata(self):
        user_data = "foobar"

        mxn = contextualization.OpenStackUserData(user_data)

        self.assertEqual("user_data", mxn.term)
        self.assertTrue(mxn.scheme.startswith(helpers._PREFIX))
        self.assertEqual(user_data, mxn.user_data)


class TestOpenStackPublicKey(base.TestCase):
    def test_os_userdata(self):
        key_name = "foobar"
        key_data = "1234"

        mxn = contextualization.OpenStackPublicKey(key_name, key_data)

        self.assertEqual("public_key", mxn.term)
        self.assertTrue(mxn.scheme.startswith(helpers._PREFIX))
        self.assertEqual(key_name, mxn.name)
        self.assertEqual(key_data, mxn.data)


class TestOSNetworkInterface(base.TestCase):
    def test_osnetwork_interface(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        n = network.NetworkResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        i = os_network.OSNetworkInterface(c, n, "00:01:02:03:04:05",
                                          "127.0.0.1", pool="foo")
        self.assertEqual('_'.join([c.id, n.id, "127.0.0.1"]), i.id)
        self.assertEqual(i.address, "127.0.0.1")
        self.assertEqual(i.interface, "eth0")
        self.assertEqual(i.mac, "00:01:02:03:04:05")
        self.assertEqual(i.state, "active")
        self.assertIsNone(i.gateway)
        self.assertEqual(network_link.NetworkInterface.kind, i.kind)
        self.assertEqual(2, len(i.mixins))
        self.assertIn(network_link.ip_network_interface, i.mixins)
        # FIXME(enolfc): this won't work without proper object comparison
        # self.assertIn(p, i.mixins)
        has_pool = False
        for m in i.mixins:
            if isinstance(m, os_network.OSFloatingIPPool):
                self.assertEqual(m.term, "foo")
                has_pool = True
                break
        self.assertTrue(has_pool)
        # contains kind and mixins attributes
        for att in network_link.NetworkInterface.kind.attributes:
            self.assertIn(att, i.attributes)
        for att in network_link.ip_network_interface.attributes:
            self.assertIn(att, i.attributes)

    def test_setters(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        n = network.NetworkResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        i = os_network.OSNetworkInterface(c, n, "00:01:02:03:04:05",
                                          "127.0.0.1")
        i.address = "192.163.1.2"
        self.assertEqual(
            "192.163.1.2", i.attributes["occi.networkinterface.address"].value)
        i.gateway = "192.163.1.1"
        self.assertEqual(
            "192.163.1.1", i.attributes["occi.networkinterface.gateway"].value)
        i.allocation = "static"
        self.assertEqual(
            "static", i.attributes["occi.networkinterface.allocation"].value)

    def test_getters(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        n = network.NetworkResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        i = os_network.OSNetworkInterface(c, n, "00:01:02:03:04:05",
                                          "127.0.0.1")
        i.attributes["occi.networkinterface.address"].value = "192.163.1.2"
        self.assertEqual("192.163.1.2", i.address)
        i.attributes["occi.networkinterface.gateway"].value = "192.163.1.1"
        self.assertEqual("192.163.1.1", i.gateway)
        i.attributes["occi.networkinterface.allocation"].value = "static"
        self.assertEqual("static", i.allocation)



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class Collection(object):
    """An OCCI Collection is used to render a set of OCCI objects.

    Depending on the rendering and the contents of the collection, there will
    be one output or another.  This class should do the magic and render the
    proper information, taking into account what is in the collection.
    """
    def __init__(self, kinds=[], mixins=[], actions=[],
                 resources=[], links=[]):

        self.kinds = kinds
        self.mixins = mixins
        self.actions = actions
        self.resources = resources
        self.links = links



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import webob.exc

from ooi.occi.core import collection
from ooi.occi.rendering import headers
from ooi.occi.rendering import text
from ooi import utils


class LocationRenderer(object):
    def __init__(self, obj):
        self.obj = obj

    def render(self, env={}):
        app_url = env.get("application_url", "")
        return utils.join_url(app_url, self.obj.location)


class CollectionRenderer(headers.CollectionRenderer):
    def render(self, env={}):
        hdr = super(CollectionRenderer, self).render(env)
        return "\n".join([l[1] for l in hdr])


def get_renderer(obj):
    """Get the correct renderer for the given object."""
    if isinstance(obj, webob.exc.HTTPException):
        return text.ExceptionRenderer(obj)
    elif isinstance(obj, collection.Collection):
        return CollectionRenderer(obj)
    elif getattr(obj, 'location', None):
        return LocationRenderer(obj)
    else:
        return text.ExceptionRenderer(obj)



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
# Copyright 2015 LIP - INDIGO-DataCloud
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.api import base
from ooi.api import helpers
from ooi.api import helpers_neutron
from ooi import exception
from ooi.occi.core import collection
from ooi.occi.infrastructure import network
from ooi.occi import validator as occi_validator
from ooi.openstack import network as os_network


def parse_validate_schema(req, scheme=None,
                          required_attr=None):
    """Parse attributes and validate scheme


    Returns attributes from request
    If scheme is specified, it validates the OCCI scheme:
     -Raises exception in case of being invalid

    :param req: request
    :param: scheme: scheme to validate
    :param: required_attr: attributes required
    """
    parser = req.get_parser()(req.headers, req.body)
    if scheme:
        attributes = parser.parse()
        validator = occi_validator.Validator(attributes)
        validator.validate(scheme)
        validator.validate_attributes(required_attr)
    else:
        attributes = parser.parse_attributes(req.headers)
    return attributes


def process_parameters(req, scheme=None,
                       required_attr=None):
    """Get attributes from request parameters

    :param req: request
    :param: scheme: scheme to validate
    :param: required_attr: attributes required
    """
    parameters = parse_validate_schema(req, scheme, required_attr)
    try:
        attributes = {}
        if 'X_PROJECT_ID' in req.headers:
            attributes["X_PROJECT_ID"] = req.headers["X_PROJECT_ID"]
        if "attributes" in parameters:
            for k, v in parameters.get("attributes", None).items():
                attributes[k.strip()] = v.strip()
        if not attributes:
            attributes = None
    except Exception:
        raise exception.Invalid
    return attributes


class Controller(base.Controller):
    def __init__(self, app=None, openstack_version=None,
                 neutron_ooi_endpoint=None):
        """Network controller initialization

        :param app: application
        :param: openstack_version: nova version
        :param: neutron_ooi_endpoint: This parameter
        indicates the Neutron endpoint to load the Neutron Helper.
        If it is None, nova-network is used.
        """

        super(Controller, self).__init__(
            app=app,
            openstack_version=openstack_version)
        if neutron_ooi_endpoint:
            self.os_helper = helpers_neutron.OpenStackNeutron(
                neutron_ooi_endpoint
            )
        else:
            self.os_helper = helpers.OpenStackHelper(
                self.app,
                self.openstack_version
            )

    @staticmethod
    def _get_network_resources(networks_list):
        """Create network instances from network in json format

        :param networks_list: networks objects provides by
        the cloud infrastructure
        """
        occi_network_resources = []
        if networks_list:
            for s in networks_list:
                n_state = s['state']
                n_id = s["id"]
                n_name = s["name"]
                n_address = s.get("address", None)
                n_ip_version = s.get("ip_version", None)
                n_gateway = s.get("gateway", None)
                s = os_network.OSNetworkResource(title=n_name,
                                                 id=n_id, state=n_state,
                                                 ip_version=n_ip_version,
                                                 address=n_address,
                                                 gateway=n_gateway)
                occi_network_resources.append(s)
        return occi_network_resources

    def index(self, req):
        """List networks

        :param req: request object
        """
        occi_networks = self.os_helper.list_networks(req)
        occi_network_resources = self._get_network_resources(
            occi_networks)

        return collection.Collection(
            resources=occi_network_resources)

    def show(self, req, id):
        """Get network details

        :param req: request object
        :param id: network identification
        """
        resp = self.os_helper.get_network_details(req, id)
        occi_network_resources = self._get_network_resources(
            [resp])
        return occi_network_resources[0]

    def create(self, req, body=None):
        """Create a network instance in the cloud

        :param req: request object
        :param body: body request (not used)
        """
        scheme = {
            "category": network.NetworkResource.kind,
            "mixins": [
                network.ip_network,
            ],
            "optional_mixins": [
                os_network.os_network
            ]
        }
        required = ["occi.core.title",
                    "occi.network.address",
                    ]
        attributes = process_parameters(req, scheme, required)
        name = attributes.get('occi.core.title')
        cidr = attributes.get('occi.network.address')
        gateway = attributes.get('occi.network.gateway', None)
        ip_version = attributes.get('org.openstack.network.ip_version', None)
        net = self.os_helper.create_network(req, name=name,
                                            cidr=cidr,
                                            gateway=gateway,
                                            ip_version=ip_version)
        occi_network_resources = self._get_network_resources([net])
        return collection.Collection(
            resources=occi_network_resources)

    def delete(self, req, id):
        """delete networks which satisfy the parameters

        :param req: current request
        :param id: identification
        """
        response = self.os_helper.delete_network(req, id)
        return response

    def run_action(self, req, id, body):
        """Run action over the network

        :param req: current request
        :param id: network identification
        :param body: body
        """
        action = req.GET.get("action", None)
        occi_actions = [a.term for a in network.NetworkResource.actions]

        if action is None or action not in occi_actions:
            raise exception.InvalidAction(action=action)
        raise exception.NotImplemented("Network actions are not implemented")


# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import webob.exc

from ooi.occi.rendering import headers


class TextRenderer(object):
    """Render OCCI objects into text.

    The text rendering is just the representation of the OCCI HTTP headers into
    text plain, so this renderer wraps around the actual header renderer and
    converts the headers into text.
    """
    def __init__(self, renderer):
        self.renderer = renderer

    def render(self, *args, **kwargs):
        """Render the OCCI object into text."""
        hdrs = self.renderer.render(*args, **kwargs)
        result = []
        for hdr in hdrs:
            result.append("%s: %s" % hdr)
        return "\n".join(result)


class ExceptionRenderer(object):
    def __init__(self, obj):
        self.obj = obj

    def render(self, *args, **kwargs):
        return self.obj.explanation


def get_renderer(obj):
    """Get the correct renderer for the given object."""
    if isinstance(obj, webob.exc.HTTPException):
        return ExceptionRenderer(obj)
    else:
        return TextRenderer(headers.get_renderer(obj))



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ooi.occi.core import link
from ooi.occi.core import mixin
from ooi.occi.core import resource
from ooi.occi.infrastructure import compute
from ooi.occi.infrastructure import network
from ooi.occi.infrastructure import network_link
from ooi.occi.infrastructure import storage
from ooi.occi.infrastructure import storage_link
from ooi.occi.infrastructure import templates
from ooi.tests import base


class TestOCCICompute(base.TestCase):
    def test_compute_class(self):
        c = compute.ComputeResource
        self.assertIn(compute.start, c.actions)
        self.assertIn(compute.stop, c.actions)
        self.assertIn(compute.suspend, c.actions)
        self.assertIn(compute.restart, c.actions)
        self.assertIn("occi.core.id", c.attributes)
        self.assertIn("occi.core.summary", c.attributes)
        self.assertIn("occi.core.title", c.attributes)
        self.assertIn("occi.compute.architecture", c.attributes)
        self.assertIn("occi.compute.cores", c.attributes)
        self.assertIn("occi.compute.hostname", c.attributes)
        self.assertIn("occi.compute.memory", c.attributes)
        self.assertIn("occi.compute.speed", c.attributes)
        self.assertIn("occi.compute.state", c.attributes)
        self.assertIn(resource.Resource.kind, c.kind.related)
        self.assertEqual(c.kind.location, "compute/")
        # TODO(aloga): We need to check that the attributes are actually set
        # after we get an object (we have to check this for this but also for
        # the other resources)

    def test_compute(self):
        id = uuid.uuid4().hex
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=id)
        self.assertEqual("foo", c.title)
        self.assertEqual(id, c.id)
        self.assertEqual("This is a summary", c.summary)
        self.assertIsNone(c.architecture)
        self.assertIsNone(c.cores)
        self.assertIsNone(c.hostname)
        self.assertIsNone(c.memory)
        self.assertIsNone(c.speed)

    def test_setters(self):
        c = compute.ComputeResource("foo")
        c.architecture = "bar"
        self.assertEqual("bar",
                         c.attributes["occi.compute.architecture"].value)
        c.cores = 5
        self.assertEqual(5, c.attributes["occi.compute.cores"].value)
        c.hostname = "foobar"
        self.assertEqual("foobar", c.attributes["occi.compute.hostname"].value)
        c.speed = 8
        self.assertEqual(8, c.attributes["occi.compute.speed"].value)
        c.memory = 4
        self.assertEqual(4, c.attributes["occi.compute.memory"].value)

    def test_getters(self):
        c = compute.ComputeResource("foo", state="baz")
        self.assertEqual("baz", c.state)
        c.attributes["occi.compute.architecture"].value = "bar"
        self.assertEqual("bar", c.architecture)
        c.attributes["occi.compute.cores"].value = 5
        self.assertEqual(5, c.cores)
        c.attributes["occi.compute.hostname"].value = "foobar"
        self.assertEqual("foobar", c.hostname)
        c.attributes["occi.compute.speed"].value = 8
        self.assertEqual(8, c.speed)
        c.attributes["occi.compute.memory"].value = 9
        self.assertEqual(9, c.memory)


class TestOCCIStorage(base.TestCase):
    def test_storage_class(self):
        s = storage.StorageResource
        self.assertIn(storage.online, s.actions)
        self.assertIn(storage.offline, s.actions)
        self.assertIn(storage.backup, s.actions)
        self.assertIn(storage.snapshot, s.actions)
        self.assertIn(storage.resize, s.actions)
        self.assertIn("occi.core.id", s.attributes)
        self.assertIn("occi.core.summary", s.attributes)
        self.assertIn("occi.core.title", s.attributes)
        self.assertIn("occi.storage.size", s.attributes)
        self.assertIn("occi.storage.state", s.attributes)
        self.assertIn(resource.Resource.kind, s.kind.related)
        self.assertEqual(s.kind.location, "storage/")
        # TODO(aloga): We need to check that the attributes are actually set
        # after we get an object (we have to check this for this but also for
        # the other resources)

    def test_storage(self):
        id = uuid.uuid4().hex
        s = storage.StorageResource("foo",
                                    summary="This is a summary",
                                    id=id)
        self.assertEqual("foo", s.title)
        self.assertEqual(id, s.id)
        self.assertEqual("This is a summary", s.summary)
        self.assertIsNone(s.size)
        self.assertIsNone(s.state)

    def test_setters(self):
        s = storage.StorageResource("foo")
        s.size = 3
        self.assertEqual(3, s.attributes["occi.storage.size"].value)

    def test_getters(self):
        s = storage.StorageResource("foo", size=5, state="foobar")
        self.assertEqual(5, s.size)
        self.assertEqual("foobar", s.state)


class TestOCCIStorageLink(base.TestCase):
    def test_storagelink_class(self):
        s = storage_link.StorageLink
        self.assertIn("occi.core.id", s.attributes)
        self.assertIn("occi.core.title", s.attributes)
        self.assertIn("occi.core.source", s.attributes)
        self.assertIn("occi.core.target", s.attributes)
        self.assertIn("occi.storagelink.mountpoint", s.attributes)
        self.assertIn("occi.storagelink.deviceid", s.attributes)
        self.assertIn("occi.storagelink.state", s.attributes)
        self.assertIn(link.Link.kind, s.kind.related)
        self.assertEqual(s.kind.location, "storagelink/")

    def test_storagelink(self):
        server_id = uuid.uuid4().hex
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=server_id)
        vol_id = uuid.uuid4().hex
        s = storage.StorageResource("bar",
                                    summary="This is a summary",
                                    id=vol_id)
        l = storage_link.StorageLink(c, s)
        link_id = '%s_%s' % (server_id, vol_id)
        self.assertEqual(link_id, l.id)
        self.assertIsNone(l.deviceid)
        self.assertIsNone(l.mountpoint)
        self.assertIsNone(l.state)

    def test_setters(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        s = storage.StorageResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        l = storage_link.StorageLink(c, s)
        l.deviceid = "/dev/vdc"
        self.assertEqual("/dev/vdc",
                         l.attributes["occi.storagelink.deviceid"].value)
        l.mountpoint = "/mnt"
        self.assertEqual("/mnt",
                         l.attributes["occi.storagelink.mountpoint"].value)

    def test_getters(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        s = storage.StorageResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        l = storage_link.StorageLink(c, s, deviceid="/dev/vdc",
                                     mountpoint="/mnt", state="foobar")
        self.assertEqual("/dev/vdc", l.deviceid)
        self.assertEqual("/mnt", l.mountpoint)
        self.assertEqual("foobar", l.state)


class TestTemplates(base.TestCase):
    def test_os_tpl(self):
        self.assertIsInstance(templates.os_tpl,
                              mixin.Mixin)
        self.assertEqual("os_tpl",
                         templates.os_tpl.term)

    def test_resource_tpl(self):
        self.assertIsInstance(templates.resource_tpl,
                              mixin.Mixin)
        self.assertEqual("resource_tpl",
                         templates.resource_tpl.term)


class TestOCCINetwork(base.TestCase):
    def test_network_class(self):
        n = network.NetworkResource
        self.assertIn(network.up, n.actions)
        self.assertIn(network.down, n.actions)
        self.assertIn("occi.core.id", n.attributes)
        self.assertIn("occi.core.summary", n.attributes)
        self.assertIn("occi.core.title", n.attributes)
        self.assertIn("occi.network.vlan", n.attributes)
        self.assertIn("occi.network.label", n.attributes)
        self.assertIn("occi.network.state", n.attributes)
        self.assertIn(resource.Resource.kind, n.kind.related)
        self.assertEqual(n.kind.location, "network/")
        # TODO(aloga): We need to check that the attributes are actually set
        # after we get an object (we have to check this for this but also for
        # the other resources)

    def test_network(self):
        id = uuid.uuid4().hex
        n = network.NetworkResource("foo",
                                    summary="This is a summary",
                                    id=id)
        self.assertEqual("foo", n.title)
        self.assertEqual(id, n.id)
        self.assertEqual("This is a summary", n.summary)
        self.assertIsNone(n.vlan)
        self.assertIsNone(n.label)
        self.assertIsNone(n.state)

    def test_setters(self):
        n = network.NetworkResource("foo")
        n.vlan = "bar"
        self.assertEqual("bar", n.attributes["occi.network.vlan"].value)
        n.label = "baz"
        self.assertEqual("baz", n.attributes["occi.network.label"].value)

    def test_getters(self):
        n = network.NetworkResource("foo", vlan="bar", label="baz",
                                    state="foobar")
        self.assertEqual("bar", n.vlan)
        self.assertEqual("baz", n.label)
        self.assertEqual("foobar", n.state)


class TestNetworkMixins(base.TestCase):
    def test_ip_network(self):
        self.assertIsInstance(network.ip_network,
                              mixin.Mixin)
        self.assertEqual("ipnetwork",
                         network.ip_network.term)
        self.assertIn("occi.network.address", network.ip_network.attributes)
        self.assertIn("occi.network.gateway", network.ip_network.attributes)
        self.assertIn("occi.network.allocation", network.ip_network.attributes)

    def test_ip_network_interface(self):
        self.assertIsInstance(network_link.ip_network_interface,
                              mixin.Mixin)
        self.assertEqual("ipnetworkinterface",
                         network_link.ip_network_interface.term)
        self.assertIn("occi.networkinterface.address",
                      network_link.ip_network_interface.attributes)
        self.assertIn("occi.networkinterface.gateway",
                      network_link.ip_network_interface.attributes)
        self.assertIn("occi.networkinterface.allocation",
                      network_link.ip_network_interface.attributes)


class TestOCCINetworkInterface(base.TestCase):
    def test_networkinterface_class(self):
        l = network_link.NetworkInterface
        self.assertIn("occi.core.id", l.attributes)
        self.assertIn("occi.core.title", l.attributes)
        self.assertIn("occi.core.source", l.attributes)
        self.assertIn("occi.core.target", l.attributes)
        self.assertIn("occi.networkinterface.interface", l.attributes)
        self.assertIn("occi.networkinterface.mac", l.attributes)
        self.assertIn("occi.networkinterface.state", l.attributes)
        self.assertIn(link.Link.kind, l.kind.related)
        self.assertEqual(l.kind.location, "networklink/")

    def test_networkinterface(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        n = network.NetworkResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        l = network_link.NetworkInterface([], c, n)
        self.assertEqual(c, l.source)
        self.assertEqual(n, l.target)
        self.assertIsNone(l.interface)
        self.assertIsNone(l.mac)
        self.assertIsNone(l.state)

    def test_setters(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        n = network.NetworkResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        l = network_link.NetworkInterface([], c, n)
        l.mac = "00:00:00:00:00:00"
        self.assertEqual("00:00:00:00:00:00",
                         l.attributes["occi.networkinterface.mac"].value)

    def test_getters(self):
        c = compute.ComputeResource("foo",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        n = network.NetworkResource("bar",
                                    summary="This is a summary",
                                    id=uuid.uuid4().hex)
        l = network_link.NetworkInterface([], c, n, interface="eth1",
                                          mac="00:01:02:03:04:05", state="foo")
        self.assertEqual("eth1", l.interface)
        self.assertEqual("00:01:02:03:04:05", l.mac)
        self.assertEqual("foo", l.state)



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections


from ooi import exception
from ooi.tests import base
from ooi.wsgi import parsers


class TestParserBase(base.TestCase):
    """Base parser tests, uses the header Parser."""

    def _get_parser(self, headers, body):
        return parsers.HeaderParser(headers, body)

    def test_kind(self):
        headers = {
            'Category': ('foo; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind"')
        }
        parser = self._get_parser(headers, None)
        res = parser.parse()
        self.assertEqual("http://example.com/scheme#foo", res["category"])
        self.assertItemsEqual(["foo"],
                              res["schemes"]["http://example.com/scheme#"])
        self.assertEqual({}, res["mixins"])
        self.assertEqual({}, res["attributes"])

    def test_missing_categories(self):
        parser = self._get_parser({}, None)
        self.assertRaises(exception.OCCIInvalidSchema,
                          parser.parse)

    def test_bad_category(self):
        headers = {
            'Category': 'foo; scheme;'
        }
        parser = self._get_parser(headers, None)
        self.assertRaises(exception.OCCIInvalidSchema,
                          parser.parse)

    def test_multiple_kinds(self):
        headers = {
            'Category': ('foo; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind", '
                         'bar; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind"')
        }
        parser = self._get_parser(headers, None)
        self.assertRaises(exception.OCCIInvalidSchema,
                          parser.parse)

    def test_mixins(self):
        headers = {
            'Category': ('foo; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind", '
                         'bar; '
                         'scheme="http://example.com/scheme#"; '
                         'class="mixin", '
                         'baz; '
                         'scheme="http://example.com/scheme#"; '
                         'class="mixin"')
        }
        parser = self._get_parser(headers, None)
        res = parser.parse()
        expected_mixins = collections.Counter(
            ["http://example.com/scheme#bar", "http://example.com/scheme#baz"])
        expected_terms = ["bar", "baz", "foo"]
        self.assertEqual(expected_mixins, res["mixins"])
        self.assertItemsEqual(expected_terms,
                              res["schemes"]["http://example.com/scheme#"])
        self.assertEqual({}, res["attributes"])

    def test_attributes(self):
        headers = {
            'Category': ('foo; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind"'),
            'X-OCCI-Attribute': 'foo="bar", baz=1234, bazonk="foo=123"',
        }
        parser = self._get_parser(headers, None)
        res = parser.parse()
        expected_attrs = {"foo": "bar", "baz": "1234", "bazonk": "foo=123"}
        self.assertEqual(expected_attrs, res["attributes"])

    def test_link(self):
        headers = {
            'Category': ('foo; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind"'),
            'Link': ('<bar>; foo="bar"; "bazonk"="foo=123"')
        }
        parser = self._get_parser(headers, None)
        res = parser.parse()
        expected_links = {"bar": {"foo": "bar", "bazonk": "foo=123"}}
        self.assertEqual(expected_links, res["links"])

    def test_invalid_link(self):
        headers = {
            'Category': ('foo; '
                         'scheme="http://example.com/scheme#"; '
                         'class="kind"'),
            'Link': ('bar; foo="bar"; "bazonk"="foo=123"')
        }
        parser = self._get_parser(headers, None)
        self.assertRaises(exception.OCCIInvalidSchema,
                          parser.parse)


class TestTextParser(TestParserBase):
    def _get_parser(self, headers, body):
        new_body = [': '.join([hdr, headers[hdr]]) for hdr in headers]
        return parsers.TextParser({}, '\n'.join(new_body))



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.api import base
import ooi.api.helpers
from ooi import exception
from ooi.occi.core import collection
from ooi.occi.infrastructure import storage
from ooi.occi import validator as occi_validator
from ooi.openstack import helpers


class Controller(base.Controller):
    def __init__(self, *args, **kwargs):
        super(Controller, self).__init__(*args, **kwargs)
        self.os_helper = ooi.api.helpers.OpenStackHelper(
            self.app,
            self.openstack_version
        )

    def index(self, req):
        volumes = self.os_helper.get_volumes(req)
        occi_storage_resources = []
        if volumes:
            for v in volumes:
                s = storage.StorageResource(title=v["displayName"], id=v["id"])
                occi_storage_resources.append(s)

        return collection.Collection(resources=occi_storage_resources)

    def show(self, req, id):
        v = self.os_helper.get_volume(req, id)

        state = helpers.vol_state(v["status"])
        st = storage.StorageResource(title=v["displayName"], id=v["id"],
                                     size=v["size"], state=state)
        return [st]

    def create(self, req, body):
        parser = req.get_parser()(req.headers, req.body)
        scheme = {"category": storage.StorageResource.kind}
        obj = parser.parse()
        validator = occi_validator.Validator(obj)
        validator.validate(scheme)

        attrs = obj.get("attributes", {})
        name = attrs.get("occi.core.title", "OCCI Volume")
        # TODO(enolfc): this should be handled by the validator
        try:
            size = attrs["occi.storage.size"]
        except KeyError:
            raise exception.Invalid()

        volume = self.os_helper.volume_create(req, name, size)

        st = storage.StorageResource(title=volume["displayName"],
                                     id=volume["id"],
                                     size=volume["size"],
                                     state=helpers.vol_state(volume["status"]))
        return collection.Collection(resources=[st])

    def _delete(self, req, vol_ids):
        for vol_id in vol_ids:
            self.os_helper.volume_delete(req, vol_id)
        return []

    # TODO(enolfc): these two methods could be in the base.Controller
    # they are identical to the ones of the Compute
    def delete(self, req, id):
        return self._delete(req, [id])

    def delete_all(self, req):
        ids = [v["id"] for v in self.os_helper.get_volumes(req)]
        return self._delete(req, ids)

    # TODO(enolfc): implement the actions.
    def run_action(self, req, id, body):
        action = req.GET.get("action", None)
        actions = [a.term for a in storage.StorageResource.actions]

        if action is None or action not in actions:
            raise exception.InvalidAction(action=action)

        raise exception.NotImplemented()



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.occi.core import mixin
from ooi.occi import helpers


class OCCIOSTemplate(mixin.Mixin):
    scheme = helpers.build_scheme("infrastructure")
    _location = "os_tpl"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("location", self._location + "/")
        super(OCCIOSTemplate, self).__init__(self.scheme, *args, **kwargs)


os_tpl = OCCIOSTemplate("os_tpl",
                        "OCCI OS Template")


class OCCIResourceTemplate(mixin.Mixin):
    scheme = helpers.build_scheme("infrastructure")
    _location = "resource_tpl"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("location", self._location + "/")
        super(OCCIResourceTemplate, self).__init__(self.scheme,
                                                   *args,
                                                   **kwargs)


resource_tpl = OCCIResourceTemplate("resource_tpl",
                                    "OCCI Resource Template")



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six.moves.urllib.parse as urlparse

_PREFIX = "http://schemas.ogf.org/occi/"


def build_scheme(category, prefix=_PREFIX):
    scheme = urlparse.urljoin(prefix, category)
    return '%s#' % scheme


def check_type(obj_list, obj_type):
    if not isinstance(obj_list, (list, tuple)):
        raise TypeError('must be a list or tuple of objects')

    if not all([isinstance(i, obj_type) for i in obj_list]):
        raise TypeError('object must be of class %s' % obj_type)


def decompose_type(type_id):
    scheme, term = type_id.split('#', 1)
    return '%s#' % scheme, term



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
# Copyright 2015 LIP - INDIGO-DataCloud
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

import routes
import routes.middleware
import webob.dec

import ooi.api.compute
import ooi.api.network
import ooi.api.network_link
from ooi.api import query
import ooi.api.storage
import ooi.api.storage_link
from ooi import config
from ooi import exception
from ooi.log import log as logging
from ooi import utils
from ooi import version
from ooi.wsgi import parsers
from ooi.wsgi import serializers

LOG = logging.getLogger(__name__)

occi_opts = [
    config.cfg.StrOpt('ooi_listen',
                      default="0.0.0.0",
                      help='The IP address on which the OCCI (ooi) API '
                           'will listen.'),
    config.cfg.IntOpt('ooi_listen_port',
                      default=8787,
                      help='The port on which the OCCI (ooi) API '
                           'will listen.'),
    config.cfg.IntOpt('ooi_workers',
                      help='Number of workers for OCCI (ooi) API service. '
                           'The default will be equal to the number of CPUs '
                           'available.'),
    # NEUTRON
    config.cfg.StrOpt('neutron_ooi_endpoint',
                      default=None,
                      help='Neutron end point which access to'
                           ' the Neutron Restful API.'),
]

CONF = config.cfg.CONF
CONF.register_opts(occi_opts)


class Request(webob.Request):
    def should_have_body(self):
        return self.method in ("POST", "PUT")

    def get_content_type(self):
        """Determine content type of the request body."""
        if not self.content_type:
            return None

        if not self.should_have_body():
            return None

        content_types = self.content_type.split(",")

        for ct in content_types:
            if ct in parsers.get_supported_content_types():
                return ct

        LOG.debug("Unrecognized Content-Type provided in request")
        raise exception.InvalidContentType(content_type=self.content_type)

    def get_best_match_content_type(self, default_match=None):
        content_types = serializers.get_supported_content_types()
        content_type = self.accept.best_match(content_types,
                                              default_match=default_match)
        if not content_type:
            LOG.debug("Unrecognized Accept Content-type provided in request")
            raise exception.InvalidAccept(content_type=self.accept)
        return content_type

    def get_parser(self):
        mtype = parsers.get_media_map().get(self.get_content_type(), "header")
        return parsers.get_default_parsers()[mtype]


class OCCIMiddleware(object):

    occi_version = "1.1"
    occi_string = "OCCI/%s" % occi_version

    @classmethod
    def factory(cls, global_conf, **local_conf):
        """Factory method for paste.deploy."""
        def _factory(app):
            conf = global_conf.copy()
            conf.update(local_conf)
            return cls(app, **local_conf)
        return _factory

    def __init__(self, application, openstack_version="/v2.1",
                 neutron_ooi_endpoint=None):
        self.application = application
        self.openstack_version = openstack_version

        if CONF.neutron_ooi_endpoint:
            self.neutron_ooi_endpoint = CONF.neutron_ooi_endpoint
        elif neutron_ooi_endpoint:
            exception.raise_deprecation_message(
                "Configuration of neutron_ooi_endpoint"
                " in api-paste.ini file is deprecated,"
                " include it in nova.conf")
            self.neutron_ooi_endpoint = neutron_ooi_endpoint
        else:
            self.neutron_ooi_endpoint = None

        self.resources = {}
        self.mapper = routes.Mapper()
        self._setup_routes()

    def _create_resource(self, controller, neutron_ooi_endpoint=None):
        if neutron_ooi_endpoint:
            return Resource(controller(self.application,
                                       self.openstack_version,
                                       neutron_ooi_endpoint))
        return Resource(controller(self.application,
                                   self.openstack_version))

    def _create_resource_network(self, controller, neutron_ooi_endpoint=None):
        if neutron_ooi_endpoint:
            return Resource(controller(
                neutron_ooi_endpoint=neutron_ooi_endpoint)
            )
        else:
            return Resource(controller(self.application,
                                       self.openstack_version))

    def _setup_resource_routes(self, resource, controller):
        path = "/" + resource
        # These two could be removed for total OCCI compliance
        self.mapper.connect(resource, path, controller=controller,
                            action="index", conditions=dict(method=["GET"]))
        self.mapper.connect(resource, path, controller=controller,
                            action="create", conditions=dict(method=["POST"]))
        # OCCI states that paths must end with a "/" when operating on pahts,
        # that are not location pahts or resource instances
        self.mapper.connect(resource, path + "/", controller=controller,
                            action="index", conditions=dict(method=["GET"]))
        self.mapper.connect(resource, path + "/", controller=controller,
                            action="create", conditions=dict(method=["POST"]))
        self.mapper.connect(resource, path + "/{id}", controller=controller,
                            action="update", conditions=dict(method=["PUT"]))
        self.mapper.connect(resource, path + "/{id}", controller=controller,
                            action="delete",
                            conditions=dict(method=["DELETE"]))
        self.mapper.connect(resource, path + "/{id}", controller=controller,
                            action="show", conditions=dict(method=["GET"]))
        # OCCI specific, delete all resources
        self.mapper.connect(path + "/", controller=controller,
                            action="delete_all",
                            conditions=dict(method=["DELETE"]))
        # Actions
        self.mapper.connect(path + "/{id}", controller=controller,
                            action="run_action",
                            conditions=dict(method=["POST"]))

    def _setup_routes(self):
        """Setup the mapper routes.

        This method should populate the mapper with the Resources
        for each of the Controllers.

        For example, if ooi.api.query contains the following:

        .. code-block:: python

            class Controller(object):
                def index(self, *args, **kwargs):
                    # Currently we do not have anything to do here
                    return None

        This method could populate the mapper as follows:

        .. code-block:: python

            self.resources["query"] = self._create_resource(query.Controller)
            self.mapper.connect("query", "/-/",
                                controller=self.resources["query"],
                                action="index")

        or if the Controller has all the CRUD operations:

        .. code-block:: python

            self.resources["servers"] = self._create_resource(query.Controller)
            self.mapper.resource("server", "servers",
                                 controller=self.resources["servers"])

        """
        self.mapper.redirect("", "/")

        self.resources["query"] = self._create_resource(
            query.Controller,
            self.neutron_ooi_endpoint
        )
        self.mapper.connect("query", "/-/",
                            controller=self.resources["query"],
                            action="index")
        # RFC5785, OCCI section 3.6.7
        self.mapper.connect("query", "/.well-known/org/ogf/occi/-/",
                            controller=self.resources["query"],
                            action="index")

        self.resources["compute"] = self._create_resource(
            ooi.api.compute.Controller)
        self._setup_resource_routes("compute", self.resources["compute"])

        self.resources["storage"] = self._create_resource(
            ooi.api.storage.Controller)
        self._setup_resource_routes("storage", self.resources["storage"])

        self.resources["storagelink"] = self._create_resource(
            ooi.api.storage_link.Controller)
        self._setup_resource_routes("storagelink",
                                    self.resources["storagelink"])

        self.resources["networklink"] = self._create_resource(
            ooi.api.network_link.Controller)
        self._setup_resource_routes("networklink",
                                    self.resources["networklink"])

        self.resources["network"] = self._create_resource_network(
            ooi.api.network.Controller, self.neutron_ooi_endpoint)
        self._setup_resource_routes("network",
                                    self.resources["network"])

    @webob.dec.wsgify(RequestClass=Request)
    def __call__(self, req):
        response = self.process_request(req)
        if not response:
            response = req.get_response(self.application)

        return self.process_response(response)

    def process_request(self, req):
        if req.user_agent:
            # FIXME(aloga): review the regexp, since it will only match the
            # first string
            match = re.search(r"\bOCCI/\d\.\d\b", req.user_agent)
            if match and self.occi_string != match.group():
                return Fault(webob.exc.HTTPNotImplemented(
                    explanation="%s not supported" % match.group()))

        match = self.mapper.match(req.path_info, req.environ)
        if not match:
            return Fault(webob.exc.HTTPNotFound())
        method = match["controller"]
        return method(req, match)

    def process_response(self, response):
        """Process a response by adding our headers."""
        server_string = "ooi/%s %s" % (version.version_string,
                                       self.occi_string)

        headers = (("server", server_string),)
        if isinstance(response, Fault):
            for key, val in headers:
                response.wrapped_exc.headers.add(key, val)
        else:
            for key, val in headers:
                response.headers.add(key, val)
        return response


class Resource(object):
    def __init__(self, controller):
        self.controller = controller
        self.default_serializers = serializers.get_default_serializers()

    def get_action_args(self, args):
        """Parse dictionary created by routes library."""
        try:
            del args['controller']
        except KeyError:
            pass

        try:
            del args['format']
        except KeyError:
            pass

        return args

    def __call__(self, request, args):
        """Control the method dispatch."""
        action_args = self.get_action_args(args)
        action = action_args.pop('action', None)
        try:
            accept = request.get_best_match_content_type()
            content_type = request.get_content_type()
        except exception.InvalidContentType as e:
            msg = e.format_message()
            return Fault(webob.exc.HTTPNotAcceptable(explanation=msg))

        body = request.body

        # Get the implementing method
        try:
            method = self.get_method(request, action,
                                     content_type, body)
        except (AttributeError, TypeError):
            return Fault(webob.exc.HTTPNotFound())
        except KeyError as ex:
            msg = "There is no such action: %s" % ex.args[0]
            return Fault(webob.exc.HTTPBadRequest(explanation=msg))

        contents = {}
        if request.should_have_body():
            # allow empty body with PUT and POST
            if request.content_length == 0:
                contents = {'body': None}
            else:
                contents["body"] = body

        action_args.update(contents)

        response = None
        try:
            with ResourceExceptionHandler():
                action_result = self.dispatch(method, request, action_args)
        except Fault as ex:
            response = ex

        # No exceptions, so create a response
        # NOTE(aloga): if the middleware returns None, the pipeline will
        # continue, but we do not want to do so, so we convert the action
        # result to a ResponseObject.
        if not response:
            if isinstance(action_result, ResponseObject):
                resp_obj = action_result
            else:
                resp_obj = ResponseObject(action_result)

            response = resp_obj.serialize(request, accept,
                                          self.default_serializers)
        return response

    def get_method(self, request, action, content_type, body):
        """Look up the action-specific method and its extensions."""

        if not self.controller:
            meth = getattr(self, action)
        else:
            meth = getattr(self.controller, action)

        return meth

    def dispatch(self, method, request, action_args):
        """Dispatch a call to the action-specific method."""
        return method(req=request, **action_args)


class ResponseObject(object):
    def __init__(self, obj, code=None, headers=None, **serializers):
        self.obj = obj
        self.serializers = serializers
        self._default_code = 200
        self._code = code
        self._headers = headers or {}
        self.serializer = None
        self.media_type = None

    def get_serializer(self, content_type, default_serializers=None):
        """Returns the serializer for the wrapped object.

        Returns the serializer for the wrapped object subject to the
        indicated content type.  If no serializer matching the content
        type is attached, an appropriate serializer drawn from the
        default serializers will be used.  If no appropriate
        serializer is available, raises InvalidContentType.
        """

        default_serializers = default_serializers or {}
        try:
            if content_type is None:
                content_type = "text/plain"

            mtype = serializers.get_media_map().get(content_type,
                                                    content_type)
            if mtype in self.serializers:
                return mtype, self.serializers[mtype]
            else:
                return mtype, default_serializers[mtype]
        except (KeyError, TypeError):
            raise exception.InvalidContentType(content_type=content_type)

    def serialize(self, request, content_type, default_serializers=None):
        if self.serializer:
            serializer = self.serializer
        else:
            _mtype, _serializer = self.get_serializer(content_type,
                                                      default_serializers)
            env = {"application_url": request.application_url + "/"}
            serializer = _serializer(env)

        response = webob.Response()
        response.status_int = self.code
        for hdr, value in self._headers.items():
            response.headers[hdr] = utils.utf8(value)
        response.headers['Content-Type'] = content_type
        response.charset = 'utf8'
        if self.obj is not None:
            headers, body = serializer.serialize(self.obj)
            if headers is not None:
                for hdr in headers:
                    response.headers.add(*hdr)
            if body:
                response.body = body

            # 204 should be used if there is no content
            if (not (headers or body) and
                    response.status_int in [200, 201, 202]):
                response.status_int = 204

        return response

    @property
    def code(self):
        """Retrieve the response status."""

        return self._code or self._default_code

    @property
    def headers(self):
        """Retrieve the headers."""

        return self._headers.copy()


class ResourceExceptionHandler(object):
    """Context manager to handle Resource exceptions.

    Used when processing exceptions generated by API implementation
    methods (or their extensions).  Converts most exceptions to Fault
    exceptions, with the appropriate logging.
    """

    def __enter__(self):
        return None

    def __exit__(self, ex_type, ex_value, ex_traceback):
        if not ex_value:
            return True

        if isinstance(ex_value, exception.OCCIException):
            raise Fault(exception.ConvertedException(
                code=ex_value.code,
                explanation=ex_value.format_message()))
        elif isinstance(ex_value, exception.NotImplemented):
            raise Fault(exception.ConvertedException(
                code=ex_value.code,
                explanation=ex_value.format_message()))
        elif isinstance(ex_value, TypeError):
            exc_info = (ex_type, ex_value, ex_traceback)
            LOG.error('Exception handling resource: %s', ex_value,
                      exc_info=exc_info)
            raise Fault(webob.exc.HTTPBadRequest())
        elif isinstance(ex_value, Fault):
            LOG.info("Fault thrown: %s", ex_value)
            raise ex_value
        elif isinstance(ex_value, webob.exc.HTTPException):
            LOG.info("HTTP exception thrown: %s", ex_value)
            raise Fault(ex_value)
        else:
            LOG.exception("Unexpected exception: %s" % ex_value)
            raise Fault(webob.exc.HTTPInternalServerError())

        # We didn't handle the exception
        return False


class Fault(webob.exc.HTTPException):
    """Wrap webob.exc.HTTPException to provide API friendly response."""

    def __init__(self, exception):
        """Create a Fault for the given webob.exc.exception."""
        self.wrapped_exc = exception
        for key, value in self.wrapped_exc.headers.items():
            self.wrapped_exc.headers[key] = str(value)
        self.status_int = exception.status_int

    @webob.dec.wsgify(RequestClass=Request)
    def __call__(self, req):
        """Generate a WSGI response based on the exception passed to ctor."""

        # Replace the body with fault details.
        code = self.wrapped_exc.status_int
        explanation = self.wrapped_exc.explanation
        LOG.debug("Returning %(code)s to user: %(explanation)s",
                  {'code': code, 'explanation': explanation})

        def_ct = "text/plain"
        content_type = req.get_best_match_content_type(default_match=def_ct)
        mtype = serializers.get_media_map().get(content_type,
                                                "text")
        serializer = serializers.get_default_serializers()[mtype]
        env = {}
        serialized_exc = serializer(env).serialize(self.wrapped_exc)
        self.wrapped_exc.content_type = content_type
        self.wrapped_exc.body = serialized_exc[1]

        # We need to specify the HEAD req.method here to be HEAD because of the
        # way that webob.exc.WSGIHTTPException.__call__ generates the response.
        # The text/occi will not have a body since it is based on headers. We
        # cannot set this earlier in the middleware, since we are calling
        # OpenStack and it will fail because the responses won't contain a
        # body.
        if content_type == "text/occi":
            req.method = "HEAD"

        return self.wrapped_exc

    def __str__(self):
        return self.wrapped_exc.__str__()



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import webob
import webob.dec
import webob.exc

from ooi.tests import base
from ooi import wsgi


@webob.dec.wsgify
def fake_app(req):
    resp = webob.Response("Foo")
    return resp


class FakeController(object):
    def index(self, *args, **kwargs):
        return None

    def create(self, req, body):
        raise webob.exc.HTTPNotImplemented()

    def delete(self, req, id):
        raise webob.exc.HTTPNotImplemented()

    def show(self, req, id):
        # Returning a ResponseObject should stop the pipepline
        # so the application won't be called.
        resp = wsgi.ResponseObject([])
        return resp


class FakeMiddleware(wsgi.OCCIMiddleware):
    def _setup_routes(self):
        self.resources["foo"] = wsgi.Resource(FakeController())
        self.mapper.resource("foo", "foos",
                             controller=self.resources["foo"])


class TestMiddleware(base.TestCase):
    def setUp(self):
        super(TestMiddleware, self).setUp()

        self.app = FakeMiddleware(fake_app)

    def test_index(self):
        result = webob.Request.blank("/foos",
                                     method="GET").get_response(self.app)
        self.assertEqual(200, result.status_code)
        self.assertEqual("", result.text)

    def test_show(self):
        result = webob.Request.blank("/foos/stop",
                                     method="GET").get_response(self.app)
        self.assertEqual(204, result.status_code)
        self.assertEqual("", result.text)

    def test_post(self):
        result = webob.Request.blank("/foos",
                                     method="POST").get_response(self.app)
        self.assertEqual(501, result.status_code)

    def test_put(self):
        # there's no put option in the FakeController
        result = webob.Request.blank("/foos/1",
                                     method="PUT").get_response(self.app)
        self.assertEqual(404, result.status_code)

    def test_delete(self):
        result = webob.Request.blank("/foos/1",
                                     method="DELETE").get_response(self.app)
        self.assertEqual(501, result.status_code)

    def test_404(self):
        result = webob.Request.blank("/bazonk").get_response(self.app)
        self.assertEqual(404, result.status_code)

    def test_empty_accept(self):
        req = webob.Request.blank("/foos",
                                  method="GET",
                                  accept=None)
        result = req.get_response(self.app)
        self.assertEqual(200, result.status_code)
        self.assertEqual("text/plain", result.content_type)

    def test_accept_all(self):
        req = webob.Request.blank("/foos",
                                  method="GET",
                                  accept="*/*")
        result = req.get_response(self.app)
        self.assertEqual(200, result.status_code)
        self.assertEqual("text/plain", result.content_type)

    def test_bad_accept(self):
        req = webob.Request.blank("/foos",
                                  method="GET",
                                  accept="foo/bazonk")
        result = req.get_response(self.app)
        self.assertEqual(406, result.status_code)

    def test_various_content_type_post(self):
        req = webob.Request.blank("/foos",
                                  method="POST",
                                  content_type="text/occi,text/plain")
        result = req.get_response(self.app)
        self.assertEqual(501, result.status_code)

    def test_various_one_bad_content_type_post(self):
        req = webob.Request.blank("/foos",
                                  method="POST",
                                  content_type="text/bazonk,text/plain")
        result = req.get_response(self.app)
        self.assertEqual(501, result.status_code)

    def test_various_bad_content_types_post(self):
        req = webob.Request.blank("/foos",
                                  method="POST",
                                  content_type="text/bazonk,text/foobar")
        result = req.get_response(self.app)
        self.assertEqual(406, result.status_code)

    def test_bad_content_type_post(self):
        req = webob.Request.blank("/foos",
                                  method="POST",
                                  content_type="foo/bazonk")
        result = req.get_response(self.app)
        self.assertEqual(406, result.status_code)

    def test_bad_content_type_put(self):
        req = webob.Request.blank("/foos",
                                  method="PUT",
                                  content_type="foo/bazonk")
        result = req.get_response(self.app)
        self.assertEqual(404, result.status_code)

    def test_bad_content_type_get(self):
        req = webob.Request.blank("/foos",
                                  method="GET",
                                  content_type="foo/bazonk")
        result = req.get_response(self.app)
        self.assertEqual(200, result.status_code)

    def test_bad_content_type_delete(self):
        req = webob.Request.blank("/foos",
                                  method="DELETE",
                                  content_type="foo/bazonk")
        result = req.get_response(self.app)
        self.assertEqual(404, result.status_code)


class TestOCCIMiddleware(base.TestCase):
    def setUp(self):
        super(TestOCCIMiddleware, self).setUp()

        self.app = wsgi.OCCIMiddleware(fake_app)



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

import mock
import six
import webob
import webob.exc

from ooi.api import compute
from ooi.api import helpers
from ooi import exception
from ooi.occi.core import collection
from ooi.occi.infrastructure import compute as occi_compute
from ooi.occi.infrastructure import network as occi_network
from ooi.occi.infrastructure import storage as occi_storage
from ooi.openstack import contextualization
from ooi.openstack import templates
from ooi.tests import base
from ooi.tests import fakes


class TestComputeController(base.TestController):
    def setUp(self):
        super(TestComputeController, self).setUp()
        self.controller = compute.Controller(mock.MagicMock(), None)

    @mock.patch.object(helpers.OpenStackHelper, "index")
    def test_index(self, m_index):
        test_servers = [
            [],
            fakes.servers[fakes.tenants["foo"]["id"]]
        ]

        for servers in test_servers:
            m_index.return_value = servers
            result = self.controller.index(None)
            expected = self.controller._get_compute_resources(servers)
            self.assertEqual(expected, result.resources)
            m_index.assert_called_with(None)

    @mock.patch.object(helpers.OpenStackHelper, "get_server")
    def test_get_server_floating_ips_no_ips(self, mock_get_server):
        mock_get_server.return_value = {}
        ret = self.controller._get_server_floating_ips(None, "foo")
        mock_get_server.assert_called_with(None, "foo")
        self.assertEqual([], ret)

    @mock.patch.object(helpers.OpenStackHelper, "get_server")
    def test_get_server_floating_ips_with_ips(self, mock_get_server):
        mock_get_server.return_value = {
            "addresses": {
                "private": [
                    {
                        "OS-EXT-IPS:type": "floating",
                        "addr": "1.2.3.4",
                    },
                    {
                        "OS-EXT-IPS:type": "fixed",
                        "addr": "10.11.12.13",
                    },
                    {
                        "OS-EXT-IPS:type": "floating",
                        "addr": "5.6.7.8",
                    },
                ]
            }
        }
        ret = self.controller._get_server_floating_ips(None, "foo")
        mock_get_server.assert_called_with(None, "foo")
        self.assertEqual(["1.2.3.4", "5.6.7.8"], ret)

    @mock.patch.object(compute.Controller, "_get_server_floating_ips")
    @mock.patch.object(helpers.OpenStackHelper, "get_floating_ips")
    @mock.patch.object(helpers.OpenStackHelper, "remove_floating_ip")
    @mock.patch.object(helpers.OpenStackHelper, "release_floating_ip")
    def test_release_floating_ips(self, mock_release, mock_remove,
                                  mock_get_floating,
                                  mock_server_floating):
        mock_server_floating.return_value = ["1.2.3.4", "5.6.7.8"]
        mock_get_floating.return_value = [
            {"ip": "1.2.3.4", "id": "bar"},
            {"ip": "5.6.7.8", "id": "baz"},
        ]
        self.controller._release_floating_ips(None, "foo")
        mock_server_floating.assert_called_with(None, "foo")
        mock_get_floating.assert_called_with(None)
        mock_remove.assert_has_calls([mock.call(None, "foo", "1.2.3.4"),
                                      mock.call(None, "foo", "5.6.7.8")])
        mock_release.assert_has_calls([mock.call(None, "bar"),
                                       mock.call(None, "baz")])

    @mock.patch.object(compute.Controller, "_delete")
    def test_delete(self, mock_delete):
        mock_delete.return_value = []
        ret = self.controller.delete(None, "foo")
        self.assertEqual([], ret)
        mock_delete.assert_called_with(None, ["foo"])

    @mock.patch.object(helpers.OpenStackHelper, "delete")
    @mock.patch.object(compute.Controller, "_release_floating_ips")
    def test_delete_ids(self, mock_release, mock_delete):
        server_ids = [uuid.uuid4().hex, uuid.uuid4().hex]
        mock_delete.return_value = None
        ret = self.controller._delete(None, server_ids)
        self.assertEqual([], ret)
        mock_delete.assert_has_calls([mock.call(None, s) for s in server_ids])
        mock_release.assert_has_calls([mock.call(None, s) for s in server_ids])

    @mock.patch.object(helpers.OpenStackHelper, "index")
    @mock.patch.object(compute.Controller, "_delete")
    def test_delete_all(self, mock_delete, mock_index):
        servers = [{"id": uuid.uuid4().hex}, {"id": uuid.uuid4().hex}]
        mock_index.return_value = servers
        mock_delete.return_value = []
        ret = self.controller.delete_all(None)
        self.assertEqual([], ret)
        mock_delete.assert_called_with(None, [s["id"] for s in servers])

    def test_run_action_none(self):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        self.assertRaises(exception.InvalidAction,
                          self.controller.run_action,
                          req,
                          None,
                          None)

    def test_run_action_invalid(self):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"], path="/foo?action=foo")
        server_uuid = uuid.uuid4().hex
        self.assertRaises(exception.InvalidAction,
                          self.controller.run_action,
                          req,
                          server_uuid,
                          None)

    @mock.patch.object(helpers.OpenStackHelper, "run_action")
    @mock.patch.object(helpers.OpenStackHelper, "get_server")
    @mock.patch("ooi.occi.validator.Validator")
    def test_run_action_start(self, m_validator, m_get_server, m_run_action):
        tenant = fakes.tenants["foo"]
#        for action in ("stop", "start", "restart", "suspend"):
        action = "start"

        state_action_map = {
            "SUSPENDED": "resume",
            "PAUSED": "unpause",
            "STOPPED": "start",
        }

        req = self._build_req(tenant["id"], path="/foo?action=start")
        for state, action in six.iteritems(state_action_map):
            req.get_parser = mock.MagicMock()
            server_uuid = uuid.uuid4().hex
            server = {"status": state}
            m_get_server.return_value = server
            m_run_action.return_value = None
            ret = self.controller.run_action(req, server_uuid, None)
            self.assertEqual([], ret)
            m_run_action.assert_called_with(mock.ANY, action, server_uuid)

    @mock.patch.object(helpers.OpenStackHelper, "get_server_volumes_link")
    @mock.patch.object(helpers.OpenStackHelper, "get_image")
    @mock.patch.object(helpers.OpenStackHelper, "get_flavor")
    @mock.patch.object(helpers.OpenStackHelper, "get_server")
    @mock.patch.object(helpers.OpenStackHelper, "get_network_id")
    def test_show(self, m_net_id, m_server, m_flavor, m_image, m_vol):
        for tenant in fakes.tenants.values():
            servers = fakes.servers[tenant["id"]]
            for server in servers:
                flavor = fakes.flavors[server["flavor"]["id"]]
                image = fakes.images[server["image"]["id"]]
                volumes = fakes.volumes.get(tenant["id"], [])
                if volumes:
                    volumes = volumes[0]["attachments"]
                net_id = fakes.networks.get(tenant["id"], [])
                if net_id:
                    net_id = net_id[0]['id']
                m_net_id.return_value = net_id
                m_server.return_value = server
                m_flavor.return_value = flavor
                m_image.return_value = image
                m_vol.return_value = volumes

                ret = self.controller.show(None, server["id"])
                # FIXME(aloga): Should we test the resource?
                self.assertIsInstance(ret[0], occi_compute.ComputeResource)
                m_server.assert_called_with(None, server["id"])
                m_flavor.assert_called_with(None, flavor["id"])
                m_image.assert_called_with(None, image["id"])
                m_vol.assert_called_with(None, server["id"])

    @mock.patch.object(helpers.OpenStackHelper, "get_server_volumes_link")
    @mock.patch.object(helpers.OpenStackHelper, "get_image")
    @mock.patch.object(helpers.OpenStackHelper, "get_flavor")
    @mock.patch.object(helpers.OpenStackHelper, "get_server")
    @mock.patch.object(helpers.OpenStackHelper, "get_network_id")
    def test_show_no_image(self, m_net_id, m_server, m_flavor, m_image, m_vol):
        for tenant in fakes.tenants.values():
            servers = fakes.servers[tenant["id"]]
            for server in servers:
                flavor = fakes.flavors[server["flavor"]["id"]]
                image = fakes.images[server["image"]["id"]]
                volumes = fakes.volumes.get(tenant["id"], [])
                if volumes:
                    volumes = volumes[0]["attachments"]
                net_id = fakes.networks.get(tenant["id"], [])
                if net_id:
                    net_id = net_id[0]['id']
                m_net_id.return_value = net_id
                m_server.return_value = server
                m_flavor.return_value = flavor
                m_image.side_effect = webob.exc.HTTPNotFound()
                m_vol.return_value = volumes

                ret = self.controller.show(None, server["id"])
                # FIXME(aloga): Should we test the resource?
                self.assertIsInstance(ret[0], occi_compute.ComputeResource)
                m_server.assert_called_with(None, server["id"])
                m_flavor.assert_called_with(None, flavor["id"])
                m_image.assert_called_with(None, image["id"])
                m_vol.assert_called_with(None, server["id"])

    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server(self, m_validator, m_net,
                           m_create):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
            },
        }
        # NOTE(aloga): the mocked call is
        # "parser = req.get_parser()(req.headers, req.body)"
        req.get_parser = mock.MagicMock()
        # NOTE(aloga): MOG!
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_create.return_value = server
        net = [{'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)
        self.assertIsInstance(ret, collection.Collection)
        m_create.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data=None,
                                    key_name=None,
                                    block_device_mapping_v2=[],
                                    networks=net)

    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server_with_context(self, m_validator, m_net,
                                        m_create):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
                "org.openstack.compute.user_data": "bazonk",
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
                contextualization.user_data.scheme: None,
                contextualization.public_key.scheme: None,
            },
        }
        # NOTE(aloga): the mocked call is
        # "parser = req.get_parser()(req.headers, req.body)"
        req.get_parser = mock.MagicMock()
        # NOTE(aloga): MOG!
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_create.return_value = server
        net = [{'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)  # noqa
        self.assertIsInstance(ret, collection.Collection)
        m_create.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data="bazonk",
                                    key_name=None,
                                    block_device_mapping_v2=[],
                                    networks=net)

    @mock.patch.object(helpers.OpenStackHelper, "keypair_create")
    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server_with_sshkeys(self, m_validator, m_net,
                                        m_server, m_keypair):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
                "org.openstack.credentials.publickey.name": "wtfoo",
                "org.openstack.credentials.publickey.data": "wtfoodata"
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
                contextualization.public_key.scheme: None,
            },
        }
        req.get_parser = mock.MagicMock()
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_server.return_value = server
        m_keypair.return_value = None
        net = [{'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)  # noqa
        self.assertIsInstance(ret, collection.Collection)
        m_keypair.assert_called_with(mock.ANY, "wtfoo",
                                     public_key="wtfoodata")
        m_server.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data=None,
                                    key_name="wtfoo",
                                    block_device_mapping_v2=[],
                                    networks=net)

    @mock.patch.object(helpers.OpenStackHelper, "keypair_delete")
    @mock.patch.object(helpers.OpenStackHelper, "keypair_create")
    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server_with_no_sshkey_name(self, m_validator, m_net,
                                               m_server, m_keypair,
                                               m_keypair_delete):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
                "org.openstack.credentials.publickey.data": "wtfoodata"
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
                contextualization.public_key.scheme: None,
            },
        }
        req.get_parser = mock.MagicMock()
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_server.return_value = server
        m_keypair.return_value = None
        net = [{'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)  # noqa
        self.assertIsInstance(ret, collection.Collection)
        m_keypair.assert_called_with(mock.ANY, mock.ANY,
                                     public_key="wtfoodata")
        m_server.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data=None,
                                    key_name=mock.ANY,
                                    block_device_mapping_v2=[],
                                    networks=net)
        m_keypair_delete.assert_called_with(mock.ANY, mock.ANY)

    def test_build_block_mapping_no_links(self):
        ret = self.controller._build_block_mapping(None, {})
        self.assertEqual([], ret)

    def test_build_block_mapping_invalid_rel(self):
        obj = {'links': {'foo': {'rel': 'bar'}}}
        ret = self.controller._build_block_mapping(None, obj)
        self.assertEqual([], ret)

    @mock.patch("ooi.api.helpers.get_id_with_kind")
    def test_build_block_mapping(self, m_get_id):
        vol_id = uuid.uuid4().hex
        image_id = uuid.uuid4().hex
        obj = {
            'links': {
                'l1': {
                    'rel': ('http://schemas.ogf.org/occi/infrastructure#'
                            'storage'),
                    'occi.core.target': vol_id,
                }
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: [image_id],
            }
        }
        m_get_id.return_value = (None, vol_id)
        ret = self.controller._build_block_mapping(None, obj)
        expected = [
            {
                "source_type": "image",
                "destination_type": "local",
                "boot_index": 0,
                "delete_on_termination": True,
                "uuid": image_id,
            },
            {
                "source_type": "volume",
                "uuid": vol_id,
                "delete_on_termination": False,
            }
        ]
        self.assertEqual(expected, ret)
        m_get_id.assert_called_with(None, vol_id,
                                    occi_storage.StorageResource.kind)

    @mock.patch("ooi.api.helpers.get_id_with_kind")
    def test_build_block_mapping_device_id(self, m_get_id):
        vol_id = uuid.uuid4().hex
        image_id = uuid.uuid4().hex
        obj = {
            'links': {
                'l1': {
                    'rel': ('http://schemas.ogf.org/occi/infrastructure#'
                            'storage'),
                    'occi.core.target': vol_id,
                    'occi.storagelink.deviceid': 'baz'
                }
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: [image_id],
            }
        }
        m_get_id.return_value = (None, vol_id)
        ret = self.controller._build_block_mapping(None, obj)
        expected = [
            {
                "source_type": "image",
                "destination_type": "local",
                "boot_index": 0,
                "delete_on_termination": True,
                "uuid": image_id,
            },
            {
                "source_type": "volume",
                "uuid": vol_id,
                "delete_on_termination": False,
                "device_name": "baz",
            }
        ]
        self.assertEqual(expected, ret)
        m_get_id.assert_called_with(None, vol_id,
                                    occi_storage.StorageResource.kind)

    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_build_block_mapping")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server_with_storage_link(self, m_validator, m_net,
                                             m_block, m_server):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
            },
        }
        req.get_parser = mock.MagicMock()
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_server.return_value = server
        m_block.return_value = "mapping"
        net = [{'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)  # noqa
        self.assertIsInstance(ret, collection.Collection)
        m_server.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data=None,
                                    key_name=mock.ANY,
                                    block_device_mapping_v2="mapping",
                                    networks=net)
        m_block.assert_called_with(req, obj)

    @mock.patch("ooi.api.helpers.get_id_with_kind")
    def test_get_network_from_req(self, m_get_id):
        net_id = uuid.uuid4().hex
        obj = {
            'links': {
                'l1': {
                    'rel': '%s%s' % (occi_network.NetworkResource.kind.scheme,
                                     occi_network.NetworkResource.kind.term),
                    'occi.core.target': net_id,
                }
            },
        }
        m_get_id.return_value = (None, net_id)
        ret = self.controller._get_network_from_req(None, obj)
        expected = [
            {
                "uuid": net_id,
            }
        ]
        self.assertEqual(expected, ret)
        m_get_id.assert_called_with(None, net_id,
                                    occi_network.NetworkResource.kind)

    @mock.patch("ooi.api.helpers.get_id_with_kind")
    def test_get_network_from_req_several_links(self, m_get_id):
        net_id_1 = uuid.uuid4().hex
        net_id_2 = uuid.uuid4().hex
        obj = {
            'links': {
                'l1': {
                    'rel': '%s%s' % (occi_network.NetworkResource.kind.scheme,
                                     occi_network.NetworkResource.kind.term),
                    'occi.core.target': net_id_1,
                },
                'l2': {
                    'rel': '%s%s' % (occi_network.NetworkResource.kind.scheme,
                                     occi_network.NetworkResource.kind.term),
                    'occi.core.target': net_id_2,
                }
            },
        }
        m_get_id.side_effect = [(None, net_id_1),
                                (None, net_id_2)
                                ]
        ret = self.controller._get_network_from_req(None, obj)
        expected = [
            {"uuid": net_id_1},
            {"uuid": net_id_2}
        ]
        self.assertEqual(expected, ret)
        self.assertEqual(2, m_get_id.call_count)
        self.assertEqual((None, mock.ANY,
                          occi_network.NetworkResource.kind),
                         m_get_id.call_args_list[1][0])
        self.assertEqual((None, mock.ANY,
                          occi_network.NetworkResource.kind),
                         m_get_id.call_args_list[0][0]
                         )
        self.assertNotEqual(m_get_id.call_args_list[0][0][1],
                            m_get_id.call_args_list[1][0][1])
        self.assertIn(m_get_id.call_args_list[0][0][1],
                      [net_id_1, net_id_2]
                      )
        self.assertIn(m_get_id.call_args_list[1][0][1],
                      [net_id_1, net_id_2]
                      )

    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server_with_network_link(self, m_validator,
                                             m_net, m_server):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
            },
        }
        req.get_parser = mock.MagicMock()
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_server.return_value = server
        net = [{'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)
        self.assertIsInstance(ret, collection.Collection)
        m_server.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data=None,
                                    key_name=mock.ANY,
                                    block_device_mapping_v2=[],
                                    networks=net)
        m_net.assert_called_with(req, obj)

    @mock.patch.object(helpers.OpenStackHelper, "create_server")
    @mock.patch.object(compute.Controller, "_get_network_from_req")
    @mock.patch("ooi.occi.validator.Validator")
    def test_create_server_with_network_link_several(self, m_validator,
                                                     m_net, m_server):
        tenant = fakes.tenants["foo"]
        req = self._build_req(tenant["id"])
        obj = {
            "attributes": {
                "occi.core.title": "foo instance",
            },
            "schemes": {
                templates.OpenStackOSTemplate.scheme: ["foo"],
                templates.OpenStackResourceTemplate.scheme: ["bar"],
            },
        }
        req.get_parser = mock.MagicMock()
        req.get_parser.return_value.return_value.parse.return_value = obj
        m_validator.validate.return_value = True
        server = {"id": uuid.uuid4().hex}
        m_server.return_value = server
        net = [{'uuid': uuid.uuid4().hex}, {'uuid': uuid.uuid4().hex}]
        m_net.return_value = net
        ret = self.controller.create(req, None)
        self.assertIsInstance(ret, collection.Collection)
        m_server.assert_called_with(mock.ANY, "foo instance", "foo", "bar",
                                    user_data=None,
                                    key_name=mock.ANY,
                                    block_device_mapping_v2=[],
                                    networks=net)
        m_net.assert_called_with(req, obj)



# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
# Copyright 2015 LIP - INDIGO-DataCloud
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.occi import helpers

_PREFIX = "http://schemas.openstack.org/"
PUBLIC_NETWORK = "PUBLIC"


def build_scheme(category):
    return helpers.build_scheme(category, prefix=_PREFIX)


def vm_state(nova_status):
    if nova_status == "ACTIVE":
        return "active"
    elif nova_status == "SUSPENDED":
        return "suspended"
    else:
        return "inactive"


# TODO(enolfc): Do really implement this.
def vol_state(nova_status):
    return "online"


def network_status(neutron_status):
    """Translate neutron network status.

    :param neutron_status: neutron status
    """
    if neutron_status == "ACTIVE":
        return "active"
    else:
        return "inactive"


# -*- coding: utf-8 -*-

# Copyright 2015 Spanish National Research Council
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ooi.occi.core import action
from ooi.occi.core import attribute as attr
from ooi.occi.core import kind
from ooi.occi.core import resource
from ooi.occi import helpers

start = action.Action(helpers.build_scheme('infrastructure/compute/action'),
                      "start", "start compute instance")

stop = action.Action(helpers.build_scheme('infrastructure/compute/action'),
                     "stop", "stop compute instance")

restart = action.Action(helpers.build_scheme('infrastructure/compute/action'),
                        "restart", "restart compute instance")

suspend = action.Action(helpers.build_scheme('infrastructure/compute/action'),
                        "suspend", "suspend compute instance")


class ComputeResource(resource.Resource):
    attributes = attr.AttributeCollection(["occi.compute.architecture",
                                           "occi.compute.cores",
                                           "occi.compute.hostname",
                                           "occi.compute.speed",
                                           "occi.compute.memory",
                                           "occi.compute.state"])
    actions = (start, stop, restart, suspend)
    kind = kind.Kind(helpers.build_scheme('infrastructure'), 'compute',
                     'compute resource', attributes, 'compute/',
                     actions=actions,
                     related=[resource.Resource.kind])

    def __init__(self, title, summary=None, id=None, architecture=None,
                 cores=None, hostname=None, speed=None, memory=None,
                 state=None, mixins=[]):

        super(ComputeResource, self).__init__(title, mixins, summary=summary,
                                              id=id)

        self.attributes["occi.compute.architecture"] = attr.MutableAttribute(
            "occi.compute.architecture", architecture)
        self.attributes["occi.compute.cores"] = attr.MutableAttribute(
            "occi.compute.cores", cores)
        self.attributes["occi.compute.hostname"] = attr.MutableAttribute(
            "occi.compute.hostname", hostname)
        self.attributes["occi.compute.speed"] = attr.MutableAttribute(
            "occi.compute.speed", speed)
        self.attributes["occi.compute.memory"] = attr.MutableAttribute(
            "occi.compute.memory", memory)
        self.attributes["occi.compute.state"] = attr.InmutableAttribute(
            "occi.compute.state", state)

    @property
    def architecture(self):
        return self.attributes["occi.compute.architecture"].value

    @architecture.setter
    def architecture(self, value):
        self.attributes["occi.compute.architecture"].value = value

    @property
    def cores(self):
        return self.attributes["occi.compute.cores"].value

    @cores.setter
    def cores(self, value):
        self.attributes["occi.compute.cores"].value = value

    @property
    def hostname(self):
        return self.attributes["occi.compute.hostname"].value

    @hostname.setter
    def hostname(self, value):
        self.attributes["occi.compute.hostname"].value = value

    @property
    def speed(self):
        return self.attributes["occi.compute.speed"].value

    @speed.setter
    def speed(self, value):
        self.attributes["occi.compute.speed"].value = value

    @property
    def memory(self):
        return self.attributes["occi.compute.memory"].value

    @memory.setter
    def memory(self, value):
        self.attributes["occi.compute.memory"].value = value

    @property
    def state(self):
        return self.attributes["occi.compute.state"].value
